%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function eukaryotes(scenario)
%% Thinking about neoproterozoic delta 13 C variability, organic
%% and inorganic, carbonate system, sulfate reduction
%%
%% Units: 10^18gr=peta kilogram, years, micro mole/kg (or /liter, I
%%        don't worry about the small difference)
%% 
%% To run this program: specify scenario number and clear all first:
%% cd ~/Snowball_initiation_and_delta13C/; clear all; eukaryotes(15)
%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%close all;
fclose('all');
if nargin<1
  disp('*** must specify an argument with number of scenario to run.');
  return
end
%% declare output variables, calculated in function r h s:
global o 

%profile on
p=set_parameters(scenario);

% This section for debugging only:
%t=0; test_K_sp_and_friends(t,p); return
%test_carbonate_system(p); return
%return

% Run some steady-state scenarios of the expanded carbonate system, with
% no progostic equations:
% calc_simple_scenarios(p);
% run_analytic_approximation_scenarios(p)

if p.do_run_model
  %% set (nondimensional) initial conditions from restart or defaults:
  [initial_conditions,restart]=set_initial_conditions(p);

  %% Integrate: can use one of ode15s, ode45, ode23s
  [t,x] = ode15s(@rhs,[0:p.integration_time/1000:p.integration_time] ...
                 ,initial_conditions,p.ode_options,p,restart);

  %% save to output file for plotting:
  save(sprintf('Output/history_files/out_scenario%2.2d.mat',p.scenario),'p','o');

  %% dimensionalize output of ode solver:
  N=length(x(1,:)); %% number of prognostic variables
  for i=1:N; x(:,i)=x(:,i)*p.prognostic_scale(i)+p.prognostic_offset(i);end

  %profile report

  %% write restart:
  if p.do_write_restart==1; 
    e=o.nt;
    restart.alpha_wthr=o.alpha_wthr(e);
    restart.F_b_inorg=o.F_b_inorg_CaCO3(e);
    last=length(t); 
    write_restart(t(last),x(last,:),p,restart); 
  end
end

plot_results(p.scenario);
examine_oxygen_consequences(p);


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function [initial_conditions,restart]=set_initial_conditions(p)
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%% solve linear problem for the initial deltas given the fluxes and
%% reservoirs, d delta/dt=A*delta+B, where delta is a two-vector:
t=0;
F_prod_c=F_prod(t,p,p.F_remin0);
A=[-F_prod_c/p.M_org_init F_prod_c/p.M_org_init; ...
   p.F_remin0/p.M_inorg_init ...
   -(F_in(t,p)+p.F_remin0)/p.M_inorg_init];
B=[F_prod_c*p.epsilon/p.M_org_init; ...
   (F_in(t,p)*p.delta_in-F_prod_c*p.epsilon)/p.M_inorg_init];
%% set initial conditions:
delta_init=-A\B;% +[0.1 0.1]';

%% default values for prognostic variables initial conditions:
initial_conditions=[delta_init' p.M_org_init p.M_inorg_init ...
                    p.Alk0 p.SO40 p.H2S0 p.Fe3p0 p.Fe2p0 p.Ca2p0];
%% augment alkalinity i.c. if sulfate is included:
if strcmp(p.carbonate_system_solver,'sulfur')==1;
  initial_conditions(p.prognostic_index.Alk)=...
      initial_conditions(p.prognostic_index.Alk)+2*p.SO40;
end

%% default values for these parameters in case restart does not
%% exist and they are still needed:
restart.alpha_wthr=p.alpha_wthr0; 
restart.F_b_inorg=p.F_b_inorg0; 

%% read restart data:
if p.do_read_restart==1; 
  [t0,x0,restart_in]=read_restart(p);
  if 0; % p.do_sulfate_remineralization==1
    fprintf(1,' *** overwriting some restart initial conditions.\n');
    %x0(p.prognostic_index.SO4_T)=p.SO40;  %% SO4_T
    %x0(p.prognostic_index.H2S_T)=p.H2S0;  %% H2S_T
  end
  if isnan(t0)==0; 
    t=t0; 
    initial_conditions=x0; 
    restart=restart_in;
  end
end

%% add an initial perturbation if so prescribed:
if p.Delta_M_inorg_initial_perturbation~=0.0
  fprintf(1,'\n\n *** adding an initial perturbation to M_inorg! \n\n');
  initial_conditions(p.prognostic_index.M_inorg)=...
      initial_conditions(p.prognostic_index.M_inorg) ...
      *(1+p.Delta_M_inorg_initial_perturbation);
end

%% nondimensionalize initial conditions:
initial_conditions=(initial_conditions-p.prognostic_offset)./p.prognostic_scale;


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function plot_results(scenario)
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%% read data file, which contains both the o. (output) and
%% p. (parameters) structures:
load(sprintf('Output/history_files/out_scenario%2.2d.mat',scenario));

b=1;
e=o.nt;
Myr=1.e6;
P=p.K_sp_pressure;
% organic burial fractions:
f=o.F_b_org./o.F_in;
f_delta=(o.F_prod-(o.F_remin+o.F_remin_SO4+o.F_remin_Fe))./o.F_in;

if min(o.F_b_org)<0; 
  fprintf(1,'\n*** found F_b_org<0! *** \n');
end

figure(1);
clf
isubplot=0; nrow=p.fig1_nrows_ncols(1); ncol=p.fig1_nrows_ncols(2); i_actual_isubplot=0;
%% note: char(97:122) corresponds to abc...z
%% Fluxes: F_in, F_prod, F_remin et al:
isubplot=isubplot+1;
if p.fig1_plot_panels(isubplot)==1;
i_actual_isubplot=i_actual_isubplot+1;
my_subplot(nrow,ncol,p.fig1_panel_order(isubplot));
factor_units=1.e3; % translate from peta kg to Peta g
factor_F_in=0.5*(max(o.F_prod(b:e))+min(o.F_prod(b:e)))/p.F_in0;
hl1=plot(o.t(b:e)/Myr,o.F_in(b:e)*factor_F_in*factor_units,'-b');
hold on
hl2=plot(o.t(b:e)/Myr,o.F_prod(b:e)*factor_units,'-.k');
%Delta_F=min(o.F_prod(b:e));
hl3=plot(o.t(b:e)/Myr,o.F_remin(b:e)*factor_units,'-r');
Delta_1_F=min(o.F_remin(b:e));
hl4=plot(o.t(b:e)/Myr,o.F_remin_SO4(b:e)*factor_units ...
         +Delta_1_F*factor_units,'--g');
hl5=plot(o.t(b:e)/Myr,o.F_remin_Fe(b:e)*factor_units ...
         +Delta_1_F*factor_units,'--c');
set(gca,'xticklabel','');
%h1=xlabel('time (Myr)');
h2=ylabel('Peta g C/yr');
h3=title(sprintf('(%s) Fluxes',char(96+i_actual_isubplot)));
set(h3,'units','normalized'); 
P=get(h3,'Position');
set(h3,'Position',[P(1) P(2)-0.02 P(3)]);
h4=legend( ...
  sprintf('F{in} \\times%3.0f',factor_F_in) ...
  ,sprintf('F{prod}') ...
  ,sprintf('F{remin,O2}')...
  ,sprintf('(*)F{remin,SO4}')...
  ,sprintf('(*)F{remin,Fe}')...
  );
set(h4,'Location','West');
set_axis_limits(gca,o.t(b:e)/Myr,[o.F_in(b:e)*factor_F_in*factor_units ...
                    ,o.F_prod(b:e)*factor_units ...
                    ,o.F_remin(b:e)*factor_units ...
                    ,(o.F_remin_SO4(b:e)+Delta_1_F)*factor_units ...
                   ,(o.F_remin_Fe(b:e)+Delta_1_F)*factor_units ...
                   ]);
set([gca,h2,h3],'fontsize',12);
set([h4],'fontsize',7);
set([hl1,hl2,hl3,hl4,hl5],'linewidth',1.5);
end

%% carbonate system:
isubplot=isubplot+1;
if p.fig1_plot_panels(isubplot)==1;
i_actual_isubplot=i_actual_isubplot+1;
factor=1.e-3;  % convert to mmole/liter for plot
my_subplot(nrow,ncol,p.fig1_panel_order(isubplot));
AlkC=o.HCO3m+2*o.CO3m2;
hl1=plot(o.t/Myr,factor*AlkC,'m');
hold on
hl2=plot(o.t/Myr,factor*o.C_T,'b');
hl3=plot(o.t/Myr,factor*o.HCO3m,'--g');
hl4=plot(o.t/Myr,factor*o.CO3m2,'r');
hl5=plot(o.t/Myr,factor*o.CO3m2_sat,'--k');
hl6=plot(o.t/Myr,factor*o.H2CO3,'c');
set_axis_limits(gca,o.t(b:e)/Myr,[factor*AlkC,factor*o.C_T,factor*o.HCO3m...
                    ,factor*o.CO3m2,factor*o.CO3m2_sat,factor*o.H2CO3])
set(gca,'xticklabel','');
%h1=xlabel('time (Myr)');
h2=ylabel('mmole/ liter');
h3=title(sprintf('(%s) Carbonate',char(96+i_actual_isubplot)));
set([gca,h2,h3],'FontSize',12);
h4=legend('Alk(C)','C(T)','HCO3-','CO3(2-)','CO3-(sat)','H2CO3*');
set([h4],'Location','West');
set([h4],'FontSize',7);
set([hl1,hl2,hl3,hl4,hl5,hl6],'linewidth',1.5);
set(h3,'units','normalized'); 
P=get(h3,'Position');
set(h3,'Position',[P(1) P(2)-0.02 P(3)]);
end

%% carbon mass and carbon isotope time series:
% carbon mass time series:
isubplot=isubplot+1;
if p.fig1_plot_panels(isubplot)==1;
  i_actual_isubplot=i_actual_isubplot+1;
  factor_units=1.e3; % Peta kg to Peta g
  my_subplot(nrow,ncol,p.fig1_panel_order(isubplot));
  hl1=line(o.t(b:e)/Myr,o.M_org(b:e)*factor_units ...
           ,'color','r','linestyle','-','linewidth',1.5);
  hl2=line(o.t(b:e)/Myr,o.M_inorg(b:e)*factor_units ...
           ,'color','b','linestyle','-','linewidth',1.5);
  set_axis_limits(gca,o.t(b:e)/Myr...
                      ,[o.M_inorg(b:e)*factor_units,o.M_org(b:e)*factor_units])
  set(gca,'xticklabel','');
  %h1=xlabel('time (Myr)');
  h2=ylabel('Peta g C');
  h3=title(sprintf('(%s) Carbon mass, \\delta13C',char(96+i_actual_isubplot)));
  set(h3,'units','normalized'); 
  P=get(h3,'Position');
  set(h3,'Position',[P(1) P(2)-0.02 P(3)]);
  set([gca,h2,h3],'fontsize',12);
  ax1=gca;
  
% isotopic time series:
  ax2 = axes('Position',get(ax1,'Position'),...
             'XAxisLocation','top',...
             'YAxisLocation','right', ...
             'Color','none',...
             'XColor','k','YColor',[0 0.5 0]);
  hl3=line(o.t(b:e)/Myr,o.delta_org(b:e)-p.epsilon,'color','r','linestyle','--','linewidth',1.5,'Parent',ax2);
  hl4=line(o.t(b:e)/Myr,o.delta_inorg(b:e),'color','b','linestyle','--','linewidth',1.5,'Parent',ax2);
  set_axis_limits(ax2,o.t(b:e)/Myr,[o.delta_inorg(b:e),o.delta_org(b:e)-p.epsilon])
  set(ax2,'xticklabel','');
  %h1=xlabel('time (Myr)');
  h2=ylabel('permil');
  set(h3,'units','normalized'); 
  P=get(h3,'Position');
  set(h3,'Position',[P(1) P(2)-0.02 P(3)]);
  h4=legend([hl1,hl2,hl3,hl4],'M org','M inorg','\delta org-\epsilon','\delta inorg','Location','West');
  set([gca,h2,h3],'fontsize',12);
  set([h4],'fontsize',8);
%    set([hl1,hl2],'linewidth',1.5);
end

%% sulfate/ sulfur species:
isubplot=isubplot+1;
if p.fig1_plot_panels(isubplot)==1;
i_actual_isubplot=i_actual_isubplot+1;
my_subplot(nrow,ncol,p.fig1_panel_order(isubplot));
myshift=28e3;
myfactor=1e-3;
myfluxfactor=1;
flux_sulfate=-1.e-6*o.d_SO4m2_dt*p.ocean_mass_kg; 
flux_sulfate_now=flux_sulfate*0+3.5e12;
[ax,hl1,hl2]=plotyy(o.t/Myr,myfactor*(o.SO4_T-myshift) ...
                  ,o.t/Myr,flux_sulfate*myfluxfactor);
% remove double tick marks on right y-axis: (reviewer 1 didn't like them...)
set(gca,'box','off'); %remove the box
set(ax(2),'XAxisLocation','top','linewidth',1,'XTickLabel',[]) %cover the top box manually
set(ax(2),'YTick',[]) %remove the ticks
% end of removing double tick marks
hl3=line(o.t/Myr,flux_sulfate_now*myfluxfactor,'color',[0 0.5 0],'linestyle','--','parent',ax(2));
set_axis_limits(ax(1),o.t(b:e)/Myr,myfactor*[o.SO4_T-myshift])
set_axis_limits(ax(2),o.t(b:e)/Myr,myfluxfactor*[flux_sulfate_now,flux_sulfate])
set(ax(1),'xticklabel','');
set(ax(2),'xticklabel','');
%h1=xlabel('time (Myr)');
h1=ylabel(ax(1),'mmole/liter');
h2=ylabel(ax(2),'mole/year');
% move right ylabel closer to the axes:
set(get(ax(2),'ylabel'),'units','normalized'); 
ylabel_position=get(get(ax(2),'ylabel'),'Position');
ylabel_position=[1.06,ylabel_position(2),ylabel_position(3)];
set(get(ax(2),'ylabel'),'Position',ylabel_position);
% end of moving ylabel.  why do I bother.
h3=title(sprintf('(%s) Sulfate',char(96+i_actual_isubplot)));
set(h3,'units','normalized'); 
P=get(h3,'Position');
set(h3,'Position',[P(1) P(2)-0.02 P(3)]);
% h4=legend('{\Delta}SO4(T)','H2S(T)','HS-','H2Saq');
h4=legend('{\Delta}SO4(T)','Flux','Flux (now)');
set(h4,'Location','NorthWest');
set([gca,h1,h2,h3],'fontsize',12);
set([h4],'fontsize',8);
set([hl1,hl2,hl3],'linewidth',1.5);
end

%% iron species:
isubplot=isubplot+1;
if p.fig1_plot_panels(isubplot)==1;
i_actual_isubplot=i_actual_isubplot+1;
my_subplot(nrow,ncol,p.fig1_panel_order(isubplot));
myfactor=1e-3;
Delta_Fe3p=o.Fe3p(1);
% convert from micromole/(year*liter) to in moles/year:
d_Fe3p_dt=-1.e-6*o.d_Fe3p_dt*p.ocean_mass_kg; 
hl1=plot(o.t(b:e)/Myr,d_Fe3p_dt(b:e),'r');
hold on
Fe3p_flux_now=d_Fe3p_dt*0+6.8e12;
% Reference value for Fe2p flux: Kump and Holland: today 4e10.
% Precambrion: hydrothermal water flux at least 4-5 times larger,
% another factor of 10 due to hotter vents:
Fe2p_flux_reference=d_Fe3p_dt*0+4e10*5*10;
hl2=plot(o.t(b:e)/Myr,Fe3p_flux_now(b:e),'LineStyle','--','color','r');
hl3=plot(o.t(b:e)/Myr,o.Fe2p_flux_pyrite_formation(b:e),'LineStyle','-','color','b');
%hl4=plot(o.t(b:e)/Myr,Fe2p_flux_reference(b:e),'LineStyle','--','color','b');
set_axis_limits(gca,o.t(b:e)/Myr,[d_Fe3p_dt(b:e),Fe3p_flux_now(b:e) ...
                   ,o.Fe2p_flux_pyrite_formation(b:e)]);
set(gca,'xticklabel','');
%h1=xlabel('time (Myr)');
h2=ylabel('mole/ year');
h3=title(sprintf('(%s) Iron fluxes',char(96+i_actual_isubplot)));
set([gca,h2,h3],'fontsize',12);
set([hl1,hl2,hl3],'linewidth',1.5);
h4=legend([hl1,hl2,hl3],'Fe3+','Fe3+(now)','Fe2+');
set(h4,'location','NorthWest');
set(h4,'fontsize',8);
set(h3,'units','normalized'); 
P=get(h3,'Position');
set(h3,'Position',[P(1) P(2)-0.02 P(3)]);
end

%% Calcium Ca(2+) and weathering:
isubplot=isubplot+1;
if p.fig1_plot_panels(isubplot)==1;
i_actual_isubplot=i_actual_isubplot+1;
my_subplot(nrow,ncol,p.fig1_panel_order(isubplot));
[ax,hl1,hl2]=plotyy(o.t(b:e)/Myr,o.Ca2p(b:e)/1000 ...
                    ,o.t(b:e)/Myr,100*o.alpha_wthr(b:e),'plot');
% remove double tick marks on right y-axis: (reviewer 1 didn't like them...)
set(gca,'box','off'); %remove the box
set(ax(2),'XAxisLocation','top','linewidth',1,'XTickLabel',[]) %cover the top box manually
set(ax(2),'YTick',[]) %remove the ticks
% end of removing double tick marks
h1=ylabel(ax(1),'mmole/ liter');
h2=ylabel(ax(2),'\alpha wthr, %');
% move right ylabel closer to the axes:
set(get(ax(2),'ylabel'),'units','normalized'); 
ylabel_position=get(get(ax(2),'ylabel'),'Position');
ylabel_position=[1.08,ylabel_position(2),ylabel_position(3)];
set(get(ax(2),'ylabel'),'Position',ylabel_position);
% end of moving ylabel.  why do I bother.
h3=title(sprintf('(%s) Ca(2+) & weathering',char(96+i_actual_isubplot)));
set(h3,'units','normalized'); 
P=get(h3,'Position');
set(h3,'Position',[P(1) P(2)-0.02 P(3)]);
set(ax(1),'xticklabel','');
set(ax(2),'xticklabel','');
set_axis_limits(ax(1),o.t(b:e)/Myr,o.Ca2p/1000)
set_axis_limits(ax(2),o.t(b:e)/Myr,100*o.alpha_wthr(b:e))
set([hl1,hl2],'linewidth',1.5);
set([gca,h1,h2,h3],'fontsize',12);
set([hl1,hl2],'linewidth',1.5);
end

%alpha_wthr_consistent=o.F_in*NaN;
%for i=b:e
%  %% value of alpha_wthr required for a consistent stady state to
%  %% exist, see LaTeX notes: 
%  alpha_wthr_consistent(i)= ...
%      1-(1-p.alpha_remin_frac)*(o.F_prod(i)/o.F_in(i));
%end
%hl1=plot(o.t(b:e)/Myr,o.alpha_wthr(b:e),'r');
%hold on
%hl2=plot(o.t(b:e)/Myr,alpha_wthr_consistent(b:e),'--b');
%h4=legend('\alpha_{wthr}','consistent');

%% burial rates:
isubplot=isubplot+1;
if p.fig1_plot_panels(isubplot)==1;
i_actual_isubplot=i_actual_isubplot+1;
factor_units=1.e3; % Peta kg to Peta g
my_subplot(nrow,ncol,p.fig1_panel_order(isubplot));
hl1=plot(o.t(b:e)/Myr,o.F_b_org(b:e)*factor_units,'r');
hold on
hl2=plot(o.t(b:e)/Myr ...
         ,(o.F_b_inorg_CaCO3(b:e)+o.F_b_inorg_FeCO3(b:e))*factor_units,'b');
hl3=plot(o.t(b:e)/Myr,o.F_b_inorg_CaCO3(b:e)*factor_units,'--g');
hl4=plot(o.t(b:e)/Myr,o.F_b_inorg_FeCO3(b:e)*factor_units,'--m');
set_axis_limits(gca,o.t(b:e)/Myr,[o.F_b_org(b:e) ...
                    ,o.F_b_inorg_CaCO3(b:e)+o.F_b_inorg_FeCO3(b:e) ...
                    ,o.F_b_inorg_CaCO3(b:e) ...
                    ,o.F_b_inorg_FeCO3(b:e) ...
                   ]*factor_units)
%set(gca,'xticklabel','');
h1=xlabel('time (Myr)');
h2=ylabel('Peta g C/yr');
h3=title(sprintf('(%s) burial rates',char(96+i_actual_isubplot)));
set(h3,'units','normalized'); 
P=get(h3,'Position');
set(h3,'Position',[P(1) P(2)-0.02 P(3)]);
h4=legend('org','inorg','CaCO3','siderite','location','NorthWest');
set([gca,h2,h3],'fontsize',12);
set([h4],'fontsize',8);
set([hl1,hl2,hl3,hl4],'linewidth',1.5);
end

%% actual burial fraction time series
isubplot=isubplot+1;
if p.fig1_plot_panels(isubplot)==1;
i_actual_isubplot=i_actual_isubplot+1;
my_subplot(nrow,ncol,p.fig1_panel_order(isubplot));
hl1=plot(o.t(b:e)/Myr,f(b:e),'r');
hold on
hl2=plot(o.t(b:e)/Myr,f_delta(b:e),'--b');
set_axis_limits(gca,o.t(b:e)/Myr,[f(b:e),f_delta(b:e)])
%set(gca,'xticklabel','');
h1=xlabel('time (Myr)');
h2=ylabel('fraction');
h3=title(sprintf('(%s) Burial fractions',char(96+i_actual_isubplot)));
set(h3,'units','normalized'); 
P=get(h3,'Position');
set(h3,'Position',[P(1) P(2)-0.02 P(3)]);
h4=legend('f','f^{\delta}','location','SouthWest');
set([gca,h2,h3],'fontsize',12);
set([h4],'fontsize',8);
set([hl1,hl2],'linewidth',1.5);
end

%% CO2 and pH:
isubplot=isubplot+1;
if p.fig1_plot_panels(isubplot)==1;
i_actual_isubplot=i_actual_isubplot+1;
my_subplot(nrow,ncol,p.fig1_panel_order(isubplot));
[ax,hl1,hl2]=plotyy(o.t/Myr,o.CO2g,o.t/Myr,o.pH,'plot');
% remove double tick marks on right y-axis: (reviewer 1 didn't like them...)
set(gca,'box','off'); %remove the box
set(ax(2),'XAxisLocation','top','linewidth',1,'XTickLabel',[]) %cover the top box manually
set(ax(2),'YTick',[]) %remove the ticks
% end of removing double tick marks
set(hl2,'LineStyle','--');
V1=axis(ax(1)); V2=axis(ax(2));
%set(gca,'xticklabel','');
h1=xlabel('time (Myr)');
h2a=ylabel(ax(1),'pCO_2 (ppm)');
h2b=ylabel(ax(2),'pH');
% reviewer 1 wanted ylabes to be 20,000 ppm rather than 2x10^4 ppm.  To do
% this, I used the property editor to set the position of the three
% panel and then prited to pdf from the command line using:
%>> fig1=figure(1);set(fig1,'PaperUnits','inches','PaperSize',[4.5 6],'PaperPosition',[0.2 0 4.5 6]); saveas(fig1,'XX.pdf');
% move right ylabel closer to the axes:
% set(get(ax(2),'ylabel'),'units','normalized'); 
% ylabel_position=get(get(ax(2),'ylabel'),'Position');
% ylabel_position=[ylabel_position(1)*1.02,ylabel_position(2),ylabel_position(3)];
%set(get(ax(2),'ylabel'),'Position',ylabel_position);
% end of moving ylabel.  why do I bother.
h3=title(sprintf('(%s) atmospheric CO2, pH',char(96+i_actual_isubplot)));
set(h3,'units','normalized'); 
P=get(h3,'Position');
set(h3,'Position',[P(1) P(2)-0.02 P(3)]);
set([ax(1),ax(2),h1,h2a,h2b,h3],'fontsize',12);
set([hl1,hl2],'linewidth',1.5);
set_axis_limits(ax(1),o.t(b:e)/Myr,o.CO2g(b:e));
set_axis_limits(ax(2),o.t(b:e)/Myr,o.pH(b:e));
end

fig1=gcf;

% ------------------------------------------------------------------------

figure(2)
clf
%% components of total alkalinity:
%isubplot=isubplot+1;
%subplot(nrow,ncol,isubplot)
AlkC=2*o.CO3m2+o.HCO3m;
AlkG=2*o.CO3m2+o.HCO3m+o.OHm-o.Hp+o.BOH4m+o.HSm+o.HSO4m+2*o.SO4m2;

hl1=plot(o.t(b:e)/Myr,2*o.CO3m2(b:e),'-r');
hold on
hl2=plot(o.t(b:e)/Myr,o.HCO3m(b:e),'-g');
hl4=plot(o.t(b:e)/Myr,o.Hp(b:e),'--m');
hl3=plot(o.t(b:e)/Myr,o.OHm(b:e),'-b');
hl5=plot(o.t(b:e)/Myr,o.BOH4m(b:e),'-c');
hl6=plot(o.t(b:e)/Myr,o.HSm(b:e),'-k');
hl7=plot(o.t(b:e)/Myr,o.HSO4m(b:e),'--r');
hl8=plot(o.t(b:e)/Myr,2*o.SO4m2(b:e),'--g');
hl9=plot(o.t(b:e)/Myr,AlkC(b:e),'--k');
hl10=plot(o.t(b:e)/Myr,AlkG(b:e),'--b');
set_axis_limits(gca,o.t(b:e)/Myr...
                ,[2*o.CO3m2(b:e),o.HCO3m(b:e)...
                  ,o.OHm(b:e),o.Hp(b:e)...
                  ,o.BOH4m(b:e),o.HSm(b:e)...
                  ,o.HSO4m(b:e),2*o.SO4m2(b:e)...
                  ,AlkC(b:e),AlkG(b:e)])
h1=xlabel('time (Myr)');
h2=ylabel('\mu mole/ liter');
h3=title('Alkalinity');
h4=legend('2CO3m2','HCO3m','Hp','OHm','BOH4m','HSm','HSO4m','2SO4m2','AlkC','AlkG');
set(h4,'Location','West');
set([gca,h1,h2,h3],'fontsize',14);
set([h4],'fontsize',10);
set([hl1,hl2,hl3,hl4,hl5,hl6,hl7,hl8,hl9,hl10],'linewidth',1.5);

fig2=gcf;

%% ------------------------------------------------------------------------

figure(3)
clf
isubplot=0;
nrow=5; ncol=2;

%% Sulfide:
isubplot=isubplot+1;
my_subplot(nrow,ncol,isubplot);
%hl1=plot(o.t/Myr,o.SO4_T-myshift,'r-');
hl2=plot(o.t/Myr,o.H2S_T_ell,'b-');
hold on
hl3=plot(o.t/Myr,o.HSm,'g--');
hl4=plot(o.t/Myr,o.H2Saq,'k-');
set_axis_limits(gca,o.t(b:e)/Myr,[o.H2S_T_ell,o.HSm,o.H2Saq])
set(gca,'xticklabel','');
%h1=xlabel('time (Myr)');
h2=ylabel('\mu mole/lit');
h3=title('Sulfide');
h4=legend('H2ST','HS-','H2Saq','location','NorthWest');
set([gca,h2,h3],'fontsize',12);
set([h4],'fontsize',8);
set([hl2,hl3,hl4],'linewidth',1.5);

%% Boron:
isubplot=isubplot+1;
my_subplot(nrow,ncol,isubplot);
hl1=plot(o.t/Myr,o.B_T,'r');
hold on
hl2=plot(o.t/Myr,o.BOH3,'g');
hl3=plot(o.t/Myr,o.BOH4m,'b');
set_axis_limits(gca,o.t(b:e)/Myr,[o.B_T,o.BOH3,o.BOH4m])
set(gca,'xticklabel','');
%h1=xlabel('time (Myr)');
h2=ylabel('\mu mole/lit');
h3=title('Boron');
h4=legend('B_T','BOH_3','BOH_4^-','location','NorthWest');
set([gca,h1,h2,h3],'fontsize',12);
set([h4],'fontsize',8);
set([hl1,hl2,hl3],'linewidth',1.5);

%% Temperature:
isubplot=isubplot+1;
my_subplot(nrow,ncol,isubplot);
hl1=plot(o.t(b:e)/Myr,o.T(b:e));
set_axis_limits(gca,o.t(b:e)/Myr,o.T(b:e))
set(gca,'xticklabel','');
%h1=xlabel('time (Myr)');
h2=ylabel('T (C)');
h3=title('Temperature');
set([gca,h2,h3],'fontsize',12);
set([hl1],'linewidth',1.5);

%% Omega:
isubplot=isubplot+1;
my_subplot(nrow,ncol,isubplot);
omega_plot=o.CO3m2./o.CO3m2_sat;
hl1=plot(o.t(b:e)/Myr,omega_plot(b:e));
set_axis_limits(gca,o.t(b:e)/Myr,omega_plot(b:e))
set(gca,'xticklabel','');
%h1=xlabel('time (Myr)');
h2=ylabel('omega');
h3=title('omega');
set([gca,h2,h3],'fontsize',12);
set([hl1],'linewidth',1.5);

%% carbon mass time series in mmole/liter:
isubplot=isubplot+1;
my_subplot(nrow,ncol,isubplot);
hl1=plot(o.t(b:e)/Myr,o.M_org(b:e)*p.Carbon_mass_to_molality*1.e-3,'r');
hold on
hl2=plot(o.t(b:e)/Myr,o.M_inorg(b:e)*p.Carbon_mass_to_molality*1.e-3,'--b');
set_axis_limits(gca,o.t(b:e)/Myr...
                ,[o.M_inorg(b:e),o.M_org(b:e)]*p.Carbon_mass_to_molality*1.e-3)
set(gca,'xticklabel','');
%h1=xlabel('time (Myr)');
h2=ylabel('mmole/ liter');
h3=title('Carbon mass (mmole/ liter)');
set(h3,'units','normalized'); 
P=get(h3,'Position');
set(h3,'Position',[P(1) P(2)-0.02 P(3)]);
h4=legend('org','inorg');
set(h4,'Location','West');
set([gca,h2,h3],'fontsize',12);
set([h4],'fontsize',8);
set([hl1,hl2],'linewidth',1.5);

%% alk eqn terms
isubplot=isubplot+1;
my_subplot(nrow,ncol,isubplot);
Alk_in=o.F_in*NaN;
for i=b:e
  Alk_in(i)=2*o.alpha_wthr(i) ...
            *o.F_in(i)*p.Carbon_mass_to_molality;
end
hl1=plot(o.t(b:e)/Myr,Alk_in(b:e),'r');
hold on
hl2=plot(o.t(b:e)/Myr,2*o.F_b_inorg_CaCO3(b:e)*p.Carbon_mass_to_molality,'--b');
set_axis_limits(gca,o.t(b:e)/Myr ...
             ,[Alk_in(b:e),2*o.F_b_inorg_CaCO3(b:e)*p.Carbon_mass_to_molality])
h1=xlabel('time (Myr)');
h2=ylabel('\mu mole/(liter yr)');
h3=title('Alk eqn terms');
h4=legend('2\alpha_{wthr}F_{in}','2F_{b,inorg,CaCO3}','location','NorthWest');
set([gca,h1,h2,h3],'fontsize',12);
set([h4],'fontsize',8);
set([hl1,hl2],'linewidth',1.5);

%% terms in delta_inorg equation:
isubplot=isubplot+1;
my_subplot(nrow,ncol,isubplot);
% evaluate three terms:
term_F_in=o.F_in.*(p.delta_in-o.delta_inorg)./o.C_T;
term_F_remin=-(o.F_remin+o.F_remin_SO4).*(o.delta_inorg+p.epsilon-o.delta_org)./o.C_T;
term_minus_F_net_prod=(o.F_prod-(o.F_remin+o.F_remin_SO4)).*p.epsilon./o.C_T;
% plot:
hl1=plot(o.t(b:e)/Myr,term_F_in(b:e),'r');
hold on
hl2=plot(o.t(b:e)/Myr,term_F_remin(b:e),'--b');
hl3=plot(o.t(b:e)/Myr,term_minus_F_net_prod(b:e),'--g');
set(gca,'xticklabel','');
%h1=xlabel('time (Myr)');
h2=ylabel('\delta/Myr');
h3=title('\delta_{inorg} terms');
set_axis_limits(gca,o.t(b:e)/Myr ...
             ,[term_F_in(b:e),term_F_remin(b:e),term_minus_F_net_prod(b:e)])
set([gca,h1,h2,h3],'fontsize',12);
set([hl1,hl2,hl3],'linewidth',1.5);

%% Fe2p, Fe3p:
isubplot=isubplot+1;
my_subplot(nrow,ncol,isubplot);
hl1=plot(o.t(b:e)/Myr,1.e-3*o.Fe2p(b:e),'r');
hold on;
hl2=plot(o.t(b:e)/Myr,1.e-3*o.Fe3p(b:e),'b');
set_axis_limits(gca,o.t(b:e)/Myr,1.e-3*[o.Fe2p(b:e),o.Fe3p(b:e)])
set(gca,'xticklabel','');
%h1=xlabel('time (Myr)');
h2=ylabel('mmole/ liter');
h3=title('Fe2p, Fe3p');
h4=legend('Fe2p','Fe3p','location','NorthWest');
set([gca,h1,h2,h3],'fontsize',12);
set([hl1,hl2],'linewidth',1.5);

%% iron oxidation and siderite:
isubplot=isubplot+1;
my_subplot(nrow,ncol,isubplot);
hl1=plot(o.t(b:e)/Myr,o.F_Fe2p_mixing_ell(b:e),'r');
hold on;
hl2=plot(o.t(b:e)/Myr,o.F_Fe2p_siderite_ell(b:e),'--b');
set_axis_limits(gca,o.t(b:e)/Myr,[o.F_Fe2p_mixing_ell(b:e),o.F_Fe2p_siderite_ell(b:e)])
%set(gca,'xticklabel','');
h1=xlabel('time (Myr)');
h2=ylabel('mmole/ liter');
h3=title('Fe2p oxidation & siderite formation');
h4=legend('mixing','siderite','location','NorthWest');
set([gca,h1,h2,h3],'fontsize',12);
set([hl1,hl2],'linewidth',1.5);

%% sulfide oxidation and pyrite:
isubplot=isubplot+1;
my_subplot(nrow,ncol,isubplot);
hl1=plot(o.t(b:e)/Myr,o.F_H2S_mixing_ell(b:e),'r');
hold on;
hl2=plot(o.t(b:e)/Myr,o.F_H2S_pyrite_ell(b:e),'--b');
set_axis_limits(gca,o.t(b:e)/Myr,[o.F_H2S_mixing_ell(b:e),o.F_H2S_pyrite_ell(b:e)])
%set(gca,'xticklabel','');
h1=xlabel('time (Myr)');
h2=ylabel('mmole/ liter');
h3=title('H2S oxidation & pyrite formation');
h4=legend('mixing','pyrite','location','NorthWest');
set([gca,h1,h2,h3],'fontsize',12);
set([hl1,hl2],'linewidth',1.5);

fig3=gcf;

% ------------------------------------------------------------------------

figure(4)
clf
isubplot=0;
nrow=2; ncol=2;

%% phase space 1:
isubplot=isubplot+1;
my_subplot(nrow,ncol,isubplot);
plot(o.delta_org(b:e),o.delta_inorg(b:e))
%%set_axis_limits(gca,o.delta_org(b:e),o.delta_inorg(b:e));
h1=xlabel('\delta_{org}');
h2=ylabel('\delta_{inorg}');
h3=title('phase plane');
set([gca,h1,h2,h3],'fontsize',12);

%% phase space 2: is the delta_org equation in a steady state
isubplot=isubplot+1;
my_subplot(nrow,ncol,isubplot);
plot(o.delta_inorg(b:e),o.delta_inorg(b:e)+p.epsilon-o.delta_org(b:e))
%%set_axis_limits(gca,o.delta_inorg(b:e),o.delta_inorg(b:e)+p.epsilon-o.delta_org(b:e));
h1=xlabel('\delta_{inorg}');
h2=ylabel('\delta_{inorg}+\epsilon_{eff}-\delta_{org}');
h3=title('steady \delta_{org}?');
set([gca,h1,h2,h3],'fontsize',12);

%% phase space 3: is the delta_inorg equation in a steady state
isubplot=isubplot+1;
my_subplot(nrow,ncol,isubplot);
plot(o.delta_inorg(b:e),...
     o.delta_inorg(b:e)-p.delta_in+f_delta(b:e).*o.epsilon(b:e))
%set_axis_limits(gca,o.delta_inorg(b:e),...
%     o.delta_inorg(b:e)-p.delta_in+f_delta(b:e).*o.epsilon(b:e));
h1=xlabel('\delta_{inorg}');
h2=ylabel('\delta_{inorg}-\delta_{in}+f^{\delta}\epsilon');
h3=title('steady \delta_{inorg}?');
set([gca,h1,h2,h3],'fontsize',12);

%% epsilon and friends:
isubplot=isubplot+1;
my_subplot(nrow,ncol,isubplot);
plot(o.t(b:e),(o.delta_inorg(b:e)-o.delta_org(b:e)),'r')
h1=xlabel('time (Myr)');
h2=ylabel(' ');
h3=title('\delta_{inorg}-\delta_{org}');
%h4=legend('\epsilon_{eff}','\epsilon+(\delta_{inorg}+\epsilon_{eff}-\delta_{org})');
set([gca,h1,h2,h3],'fontsize',12);


fig4=gcf;
% ------------------------------------------------------------------------
set(fig1, 'PaperUnits', 'inches'); set(fig1, 'PaperSize' ...
                     , [p.fig1_page_size_inches(1) p.fig1_page_size_inches(2)]);
set(fig1, 'PaperPosition'...
          , [0 0 p.fig1_page_size_inches(1) p.fig1_page_size_inches(2)]); % [left, bottom, width, height];
set(fig2, 'PaperUnits', 'inches'); set(fig2, 'PaperSize', [8 5]);
set(fig2, 'PaperPosition', [0 0 8 5]); % [left, bottom, width, height];
set(fig3, 'PaperUnits', 'inches'); set(fig3, 'PaperSize', [5 5]);
set(fig3, 'PaperPosition', [0 0 5 5]); % [left, bottom, width, height];
set(fig4, 'PaperUnits', 'inches'); set(fig4, 'PaperSize', [5 5]);
set(fig4, 'PaperPosition', [0 0 5 5]); % [left, bottom, width, height];
if p.do_save_figures_as_pdf==1
  saveas(fig1,sprintf('Output/Figures_pdf/delta13C-scenario%2.2da.pdf',p.scenario));
  saveas(fig2,sprintf('Output/Figures_pdf/delta13C-scenario%2.2db.pdf',p.scenario));
  saveas(fig3,sprintf('Output/Figures_pdf/delta13C-scenario%2.2dc.pdf',p.scenario));
  saveas(fig4,sprintf('Output/Figures_pdf/delta13C-scenario%2.2dd.pdf',p.scenario));
end
if p.do_save_figures_as_Fig_files
  saveas(fig1,sprintf('Output/Figures_Fig_files/delta13C-scenario%2.2da.fig',p.scenario));
  saveas(fig2,sprintf('Output/Figures_Fig_files/delta13C-scenario%2.2db.fig',p.scenario));
  saveas(fig3,sprintf('Output/Figures_Fig_files/delta13C-scenario%2.2dc.fig',p.scenario));
  saveas(fig4,sprintf('Output/Figures_Fig_files/delta13C-scenario%2.2dd.fig',p.scenario));
end


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function set_axis_limits(h,x,y)
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%% Set axis limits tighter around min/max of y data than the
%% typical Matlab default.
%% If plotting more than one array using more than one plot call, use:
%% set_axis_limits(x,[y1,y2])
%% If y1,y2 are column vectors, use
%% set_axis_limits(x,[y1',y2'])
%% If wish to add min/max values in addition to the data (e.g. if
%% data are a constant, and a wider range is desired):
%% set_axis_limits(x,[ymin,ymax,y1,y2])

%% set y limits:
y=reshape(y,1,[]);
V=axis; 
a1=min(y); 
a2=max(y);
V(3)=a1-(a2-a1)*0.15; 
V(4)=a2+(a2-a1)*0.15;
if V(3)==V(4); 
  V(3)=a1*0.9;
  V(4)=a2*1.1;
  if V(3)==0.0
    V(3)=-1;
    V(4)=1;
  end
end
%% set x limits:
V(1)=x(1);
V(2)=x(length(x));

axis(h,V);
set(h,'YTickMode','auto');


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function Pos=my_subplot(nrows,ncols,ifig)
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Replace Matlab's subplot, to allow a better control of spacing
% between panels.  Eli, June 12, 2007.

if ifig>nrows*ncols;
  fprintf('** problem in my_subplot: ifig=%d > nrows*ncols=%d*%d=%d.\n'...
          ,ifig,nrows,ncols,nrows*ncols);
  disp('pausing.');
  pause
end

%fprintf(1,'ir,jc,nx_panels,ny_panels=%d, %d, %d, %d\n',ir,jc,nx_panels,ny_panels)
% left and right margins of page:
left_margin=0.12; 
right_margin=0;
% top margin of page:
y_top_margin=0.05;
% bottom margin of page:
y_bot_margin=0.1;
% left and right margins of each panel (relative to panel width):
dx=0.2;
% top and bottom margins of each panel (relative to panel height):
dy=0.22;
% width and height of entire plot on page:
W=1-(left_margin+right_margin); 
H=1-(y_top_margin+y_bot_margin);
% separation in x and y between panels:
xsep=W/ncols; ysep=H/nrows;
% width and height of each pannel:
width=xsep*(1-dx); height=ysep*(1-dy);
% indices of row and column of the panel being drawn:
ir=floor((ifig-1)/ncols)+1; jc=mod(ifig,ncols); if jc==0; jc=ncols; end
% position:
left=left_margin+xsep*(jc-1); bottom=(1-y_top_margin)-ysep*ir;
Pos=[left bottom width height];
axes('Position',Pos);



%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function answer = rhs(t,x,p,restart)
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%% provides the rhs of the time-dependent (prognostic) equations.

%% save carbonate system solution between calls to this function for the
%% iterative solution of the carbonate system:
persistent pH Hp OHm CO2g H2CO3 HCO3m CO3m2 BOH3 BOH4m HSO4m SO4m2 H2Saq HSm
persistent nt % time step
%% declare global output variable:
global o

if 0 %% For debugging
  format long
  fprintf('in rhs, t=%d\n',t);
  disp('in rhs:, x=');
  disp(x);
  pause
end

%% get prognostic variables from the state vector:
%% nondim=(dim-offset)/scale
%% dim=nondim*scale+offset
%% d(nondim)/dt=(d(dim)/dt)/scale
n=p.prognostic_index.delta_org;
delta_org=  x(n)*p.prognostic_scale(n)+p.prognostic_offset(n);
n=p.prognostic_index.delta_inorg;
delta_inorg=x(n)*p.prognostic_scale(n)+p.prognostic_offset(n);
n=p.prognostic_index.M_org;
M_org=      x(n)*p.prognostic_scale(n)+p.prognostic_offset(n);
n=p.prognostic_index.M_inorg;
M_inorg=    x(n)*p.prognostic_scale(n)+p.prognostic_offset(n);
n=p.prognostic_index.Alk;
Alk=        x(n)*p.prognostic_scale(n)+p.prognostic_offset(n);
n=p.prognostic_index.SO4_T;
SO4_T=      x(n)*p.prognostic_scale(n)+p.prognostic_offset(n);
n=p.prognostic_index.H2S_T;
H2S_T_ell=  x(n)*p.prognostic_scale(n)+p.prognostic_offset(n);
n=p.prognostic_index.Fe3p;
Fe3p=       x(n)*p.prognostic_scale(n)+p.prognostic_offset(n);
n=p.prognostic_index.Fe2p;
Fe2p=       x(n)*p.prognostic_scale(n)+p.prognostic_offset(n);
n=p.prognostic_index.Ca2p;
Ca2p=       x(n)*p.prognostic_scale(n)+p.prognostic_offset(n);

%% ---------

T=temperature(t,p);
S=p.Salinity0;
P=p.K_sp_pressure;
%% ---------

%% calculate carbonate saturation using Ca2p: 
CO3m2_sat=K_sp(p,P,T,S)/Ca2p;
  
%% ---------

%% solve for the carbonate system variables given the values of
%% the above state variables.  This is needed below for the
%% sediment dissolution term in the alkalinity equation:
C_T=M_inorg*1.e15*p.micromole_per_kg_C/p.ocean_mass_kg;
B_T=p.B_T0;
if isempty(Hp);
  init_guess=p.carb_sulf_init_guess; 
else; 
  init_guess=[Hp,OHm,CO2g,H2CO3,HCO3m,CO3m2,BOH3,BOH4m,HSO4m,SO4m2,H2Saq,HSm]; 
end
if     strcmp(p.carbonate_system_solver,'standard')==1
  [pH,Hp,OHm,CO2g,H2CO3,HCO3m,CO3m2,BOH3,BOH4m,HSO4m,SO4m2,H2Saq,HSm] ...
    =solve_carbonate_system_carbonate_only_iterative(t,p,Alk,C_T,B_T,SO4_T,H2S_T_ell,T,S,init_guess);
elseif strcmp(p.carbonate_system_solver,'sulfur')==1
  [pH,Hp,OHm,CO2g,H2CO3,HCO3m,CO3m2,BOH3,BOH4m,HSO4m,SO4m2,H2Saq,HSm] ...
    =solve_carbonate_system_with_sulfur(t,p,Alk,C_T,B_T,SO4_T,H2S_T_ell,T,S,init_guess);
elseif strcmp(p.carbonate_system_solver,'approx')==1
  [pH,Hp,OHm,CO2g,H2CO3,HCO3m,CO3m2,BOH3,BOH4m,HSO4m,SO4m2,H2Saq,HSm] ...
    =solve_carbonate_system_approx(t,p,Alk,C_T,B_T,SO4_T,H2S_T_ell,T,S,init_guess);
else
  disp('*** unrecognized carbonate solver, pausing.')
  pause
end

%  t,p,Alk,C_T,B_T,SO4_T,H2S_T_ell,T,S
%  pH,Hp,OHm,CO2g,H2CO3,HCO3m,CO3m2,BOH3,BOH4m,HSO4m,SO4m2,H2Saq,HSm
%  AlkC=HCO3m+2*CO3m2
%  pause  

%% ---------

%% check for unphysical values for state variables:
if min([M_org,M_inorg,Alk,SO4_T,Ca2p,Fe3p])<0
  fprintf(1,'*** negative state variable!  pausing.  t=%gMyr\n',t/1.e6)
  for i=1:length(x);
    fprintf('%s=%g;\n',p.prognostic_label(i,:),x(i)*p.prognostic_scale(i)+p.prognostic_offset(i)); 
  end
  pause
end

%% check for unphysical values for carbonate system:
if min([CO2g,H2CO3,OHm,Hp,pH,HCO3m,CO3m2,BOH3,BOH4m])<0
  fprintf(1,['*** negative carbonate variable!    t=%gMyr,\n' ...
             ' Alk=%g,\n C_T=%g,\n B_T=%g,\n SO4_T=%g,\n H2S_T_ell=%g,\n ' ...
             'CO2g=%g,\n H2CO3=%g,\n OHm=%g,\n Hp=%g,\n pH=%g,\n ' ...
             'HCO3m=%g,\n CO3m2=%g,\n BOH3=%g,\n BOH4m=%g\n pausing.\n'] ...
          ,t/1.e6,Alk,C_T,B_T,SO4_T,H2S_T_ell,CO2g,H2CO3,OHm,Hp,pH, ...
          HCO3m,CO3m2,BOH3,BOH4m);  
  pause
end

%% ---------

%% convert F_remin and friends to micromol C/(liter*year), see
%% LaTeX notes: 
F_prod_c=F_prod(t,p,F_remin(t,p));
F_prod_ell=F_prod_c*p.Carbon_mass_to_molality;
F_remin_c=F_remin(t,p);
F_remin_ell=F_remin_c*p.Carbon_mass_to_molality;
F_b_org_c=F_b_org(t,p);
F_b_org_ell=F_b_org_c*p.Carbon_mass_to_molality;
F_b_inorg_c=F_b_inorg(t,p,CO3m2,CO3m2_sat,restart);
F_b_inorg_ell=F_b_inorg_c*p.Carbon_mass_to_molality;
F_in_c=F_in(t,p);
F_in_ell=F_in_c*p.Carbon_mass_to_molality;
F_in_Ca2p_ell=p.F_in_Ca2p_ell0;
F_ep_c=F_ep(t,p);
F_ep_ell=F_ep_c*p.Carbon_mass_to_molality;
  
  
%% specify sulfate remineralization flux:
if p.do_sulfate_remineralization==1;
  F_remin_SO4_c=F_remin_SO4(t,p);
  F_remin_SO4_ell=F_remin_SO4_c*p.Carbon_mass_to_molality;
else
  F_remin_SO4_c=0;
  F_remin_SO4_ell=0;
end

%% specify iron remineralization flux:
if p.do_iron_remineralization==1;
  F_remin_Fe_c=F_remin_Fe(t,p);
  F_remin_Fe_ell=F_remin_Fe_c*p.Carbon_mass_to_molality;
else
  F_remin_Fe_c=0;
  F_remin_Fe_ell=0;
end

%% ---------
if p.do_constant_Alk==1
  FACTOR_Alk=0;
else
  FACTOR_Alk=1;
end

if p.do_prognostic_Ca2p==1
  FACTOR_Ca2p=1;
else
  FACTOR_Ca2p=0;
end

if p.do_H2S_mixing_and_oxidation==1
  F_H2S_mixing_ell_c=F_H2S_mixing_ell(t,p,H2S_T_ell);
else
  F_H2S_mixing_ell_c=0;
end

if p.do_Fe2p_mixing_and_oxidation==1
  %% *MICRO* moles per liter of Fe2p diffusing into aerobic zone per unit time:
  F_Fe2p_mixing_ell_c=F_Fe2p_mixing_ell(t,p,Fe2p);
  %% total moles of Fe2p diffusing into aerobic zone per unit time::
  F_Fe2p_mixing_total_moles_Fe_c=F_Fe2p_mixing_ell_c*p.ocean_mass_kg*1.e6;
else
  F_Fe2p_mixing_ell_c=0;
  F_Fe2p_mixing_total_moles_Fe_c=0;
end

if p.do_pyrite_formation==1
  F_H2S_pyrite_ell_c=F_H2S_pyrite_ell(t,p,H2S_T_ell);
else
  F_H2S_pyrite_ell_c=0;
end

%% Implied Fe2p flux needed for pyrite formation:
%% F_H2S_pyrite_ell_c is given as mols of sulfur, so need a factor
%% of 0.5 to convert to iron, and then to get total mols per year,
%% multiply by ocean volume in liters, and convert from micro mol
%% to mol:
Fe2p_flux_pyrite_formation=1.e-6*0.5*F_H2S_pyrite_ell_c*p.ocean_mass_kg;

if p.do_siderite_formation==1
  F_Fe2p_siderite_ell_c=F_Fe2p_siderite_ell(t,p,Fe2p);
else
  F_Fe2p_siderite_ell_c=0;
end

F_b_inorg_total_c=F_b_inorg_c+F_Fe2p_siderite_ell_c/p.Carbon_mass_to_molality;
F_prod_T=F_prod_c;
F_remin_T=F_remin_c+F_remin_SO4_c+F_remin_Fe_c;

%% initialize rhs array:
Nprognostic=length(p.prognostic_index);
answer=zeros(Nprognostic,1)*NaN;


%% -------------------------------------------------------
%% Finally, specify rhs of all prognostic equations:
%% -------------------------------------------------------

%% delta_org:
answer(p.prognostic_index.delta_org)= ...
    (F_prod_T*(delta_inorg+p.epsilon-delta_org))/M_org;
%% delta_inorg:
answer(p.prognostic_index.delta_inorg)= ...
    (F_in_c*(p.delta_in-delta_inorg) ...
     -F_remin_T*(delta_inorg+p.epsilon-delta_org) ...
     -(F_prod_T-F_remin_T)*p.epsilon ...
     )/M_inorg;
%% M_org:
answer(p.prognostic_index.M_org)= ...
    (F_prod_T-F_remin_T-F_b_org_c);
%% M_inorg:
answer(p.prognostic_index.M_inorg)= ...
    (F_in_c+F_remin_T-F_prod_T-F_b_inorg_total_c);
%% Alk:
answer(p.prognostic_index.Alk)= ...
    FACTOR_Alk*(2*alpha_wthr(t,p,CO2g,restart)*F_in_ell ... 
                -2*F_b_inorg_ell ...
                -F_H2S_pyrite_ell_c ...
                +8*F_remin_Fe_ell ...
                -2*F_Fe2p_mixing_ell_c ...
                -2*F_Fe2p_siderite_ell_c ...
                );
%% SO4_T:
answer(p.prognostic_index.SO4_T)= ...
    (-0.5*F_remin_SO4_ell+F_H2S_mixing_ell_c);
d_SO4m2_dt_dimensional=answer(p.prognostic_index.SO4_T);
%% H2S_T_ell:
answer(p.prognostic_index.H2S_T)= ...
    (+0.5*F_remin_SO4_ell-F_H2S_mixing_ell_c ...
     -F_H2S_pyrite_ell_c);
%% Fe3p:
answer(p.prognostic_index.Fe3p)= ...
    (-4*F_remin_Fe_ell+F_Fe2p_mixing_ell_c);
d_Fe3p_dt_dimensional=answer(p.prognostic_index.Fe3p);
%% Fe2p:
    %%XX note that pyrite formation is not included here!  Instead,
    %% I calculate a diagnostic for the implied Fe(II) flux due
    %% to pyrite formation.
answer(p.prognostic_index.Fe2p)= ...
    (4*F_remin_Fe_ell-F_Fe2p_mixing_ell_c ...
     -F_Fe2p_siderite_ell_c);
%% Ca2p:
answer(p.prognostic_index.Ca2p)= ...
    FACTOR_Ca2p*(F_in_Ca2p_ell-F_b_inorg_ell);

%% apply prognostic variable scaling to calculated rhs:
%% nondim=(dim-offset)/scale
%% dim=nondim*scale+offset
%% d(nondim)/dt=(d(dim)/dt)/scale
answer=answer'./p.prognostic_scale';

%% save diagnostics to output variable:
% first, advance time step used for saving diagnostics.
if isempty(nt); nt=1; else; nt=nt+1; end
if nt==1; init=1; else; init=0; end
o.max_nt=15000;
if nt>o.max_nt; 
  fprintf('*** nt>o.max_nt, nt=%d; o.max_nt=%d; t=%g\n',nt,o.max_nt,t);
  pause
end
%% matlab sometimes makes a trial step, and then goes back to get
%% more accurate results, take care of this by not saving trial
%% steps in output arrays:
%%fprintf(1,'%g, %d; ',t, nt);
while nt>2 && t<=o.t(nt-1);
  nt=nt-1;
  %%fprintf(1,'\n !! \n');
end
o.nt=nt;

if init; ZZ=zeros(o.max_nt,1)*NaN; end
if init; o.Alk=ZZ; end;             		o.Alk(nt)=Alk;
if init; o.BOH3=ZZ; end;            		o.BOH3(nt)=BOH3;
if init; o.BOH4m=ZZ; end;           		o.BOH4m(nt)=BOH4m;
if init; o.B_T=ZZ; end;             		o.B_T(nt)=B_T;
if init; o.CO2g=ZZ; end;            		o.CO2g(nt)=CO2g;
if init; o.CO3m2=ZZ; end;           		o.CO3m2(nt)=CO3m2;
if init; o.CO3m2_sat=ZZ; end;       		o.CO3m2_sat(nt)=CO3m2_sat;
if init; o.C_T=ZZ; end;             		o.C_T(nt)=C_T;
if init; o.Ca2p=ZZ; end;            		o.Ca2p(nt)=Ca2p;
if init; o.F_H2S_mixing_ell=ZZ; end;    	o.F_H2S_mixing_ell(nt)=F_H2S_mixing_ell_c;
if init; o.F_Fe2p_mixing_ell=ZZ; end;    	o.F_Fe2p_mixing_ell(nt)=F_Fe2p_mixing_ell_c;
if init; o.F_Fe2p_siderite_ell=ZZ; end;    	o.F_Fe2p_siderite_ell(nt)=F_Fe2p_siderite_ell_c;
if init; o.F_H2S_pyrite_ell=ZZ; end;    	o.F_H2S_pyrite_ell(nt)=F_H2S_pyrite_ell_c;
if init; o.F_b_inorg_CaCO3=ZZ; end; 		o.F_b_inorg_CaCO3(nt)=F_b_inorg_c;
if init; o.F_b_inorg_FeCO3=ZZ; end; 		o.F_b_inorg_FeCO3(nt)=F_Fe2p_siderite_ell_c/p.Carbon_mass_to_molality;
if init; o.F_b_org=ZZ; end;         		o.F_b_org(nt)=F_b_org_c;
if init; o.F_in=ZZ; end;            		o.F_in(nt)=F_in_c;
if init; o.F_prod=ZZ; end;          		o.F_prod(nt)=F_prod_c;
if init; o.F_remin_SO4=ZZ; end;     		o.F_remin_SO4(nt)=F_remin_SO4_c;
if init; o.F_remin_Fe=ZZ; end;      		o.F_remin_Fe(nt)=F_remin_Fe_c;
if init; o.F_remin=ZZ; end;         		o.F_remin(nt)=F_remin_c;
if init; o.F_in_Ca2p=ZZ; end;       		o.F_in_Ca2p_ell(nt)=F_in_Ca2p_ell;
if init; o.H2CO3=ZZ; end;           		o.H2CO3(nt)=H2CO3;
if init; o.H2S_T_ell=ZZ; end;       		o.H2S_T_ell(nt)=H2S_T_ell;
if init; o.H2Saq=ZZ; end;           		o.H2Saq(nt)=H2Saq;
if init; o.HCO3m=ZZ; end;           		o.HCO3m(nt)=HCO3m;
if init; o.HSO4m=ZZ; end;           		o.HSO4m(nt)=HSO4m;
if init; o.HSm=ZZ; end;             		o.HSm(nt)=HSm;
if init; o.Hp=ZZ; end;              		o.Hp(nt)=Hp;
if init; o.M_inorg=ZZ; end;         		o.M_inorg(nt)=M_inorg;
if init; o.M_org=ZZ; end;           		o.M_org(nt)=M_org;
if init; o.OHm=ZZ; end;             		o.OHm(nt)=OHm;
if init; o.S=ZZ; end;               		o.S(nt)=S;
if init; o.SO4_T=ZZ; end;           		o.SO4_T(nt)=SO4_T;
if init; o.SO4m2=ZZ; end;           		o.SO4m2(nt)=SO4m2;
if init; o.d_SO4m2_dt=ZZ; end;       		o.d_SO4m2_dt(nt)=d_SO4m2_dt_dimensional;
if init; o.Fe3p=ZZ; end;            		o.Fe3p(nt)=Fe3p;
if init; o.d_Fe3p_dt=ZZ; end;       		o.d_Fe3p_dt(nt)=d_Fe3p_dt_dimensional;
if init; o.Fe2p_flux_pyrite_formation=ZZ; end; 	o.Fe2p_flux_pyrite_formation(nt)=Fe2p_flux_pyrite_formation;
if init; o.Fe2p=ZZ; end;            		o.Fe2p(nt)=Fe2p;
if init; o.T=ZZ; end;               		o.T(nt)=T;
if init; o.alpha_wthr=ZZ; end;      		o.alpha_wthr(nt)=alpha_wthr(t,p,CO2g,restart);
if init; o.delta_inorg=ZZ; end;     		o.delta_inorg(nt)=delta_inorg;
if init; o.delta_org=ZZ; end;       		o.delta_org(nt)=delta_org;
if init; o.epsilon=ZZ; end;         		o.epsilon(nt)=p.epsilon;
if init; o.pH=ZZ; end;              		o.pH(nt)=pH;
if init; o.t=ZZ; end;               		o.t(nt)=t;

if 0 %% For debugging
  format long
  fprintf('exiting rhs, t=%d\n',t);
  disp('rhs=');
  disp(answer);
  pause
end



%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function ode_output(t,y,status) 
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%% print output on the progress of the solution  
  if status=='init'
    fprintf(1,'progress: ')
  elseif isempty(status)
    fprintf(1,'.')
  elseif status=='done'
    fprintf(1,'. \n')
  end


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function out=H(t,p);
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%% time dependence of the change in export production and friends
%% in the specified scenario

if strcmp(p.H_form,'gaussian')
  out=exp(-(t-p.t0)^2/p.tau_ep^2);
elseif strcmp(p.H_form,'step')
    out=(sqrt(pi)/2)* ...
        0.5*(1+tanh((t-(p.t0-p.tau_ep))/0.01e6)) .*  ...
        0.5*(1+tanh(((p.t0+p.tau_ep)-t)/0.01e6));
else
  disp('*** Error: unknown form for H(t), ^C to quit.')
  pause
end
  
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function out=F_ep(t,p);
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
out=p.F_ep0*(1+p.Delta_ep*H(t,p));


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function out=F_remin(t,p);
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% aerobic remineralization
out=p.F_remin0-p.beta_scenario*p.Delta_ep*p.F_ep0*H(t,p);

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function a=F_prod(t,p,F_remin);
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% aerobic productivity in carbon weight per unit time.
a=p.F_prod0-p.gamma_scenario*p.Delta_ep*p.F_ep0*H(t,p);


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function out=F_remin_SO4(t,p);
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%% moles carbon remineralized per unit time via sulfate reduction
if p.do_sulfate_remineralization==1
  out=p.alpha_scenario*p.Delta_ep*p.F_ep0*H(t,p);
else
  out=0;
end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function out=F_remin_Fe(t,p);
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%% moles carbon remineralized per unit time via iron reduction
if p.do_iron_remineralization==1
  out=p.eta_scenario*p.Delta_ep*p.F_ep0*H(t,p);
else
  out=0;
end


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function a=F_in(t,p);
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% inorganic input from weathering and volcanoes in carbon weight per
% unit time.
a=p.F_in0*(1+p.Delta_F_in*H(t,p));


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function a=F_b_inorg(t,p,CO3m2,CO3m2_sat,restart);
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
if p.do_CaCO3_dissolution==1
  Omega=CO3m2/CO3m2_sat;
  dOmega=(Omega-p.Omega0);
  a=(p.M_inorg_init/p.tau_CaCO3_dissolution) ...
    *sign(dOmega)*abs(dOmega)^p.CaCO3_exponent;
else
  %% use value from first time step:  (do this only when starting
  %% from steady state)
  a=restart.F_b_inorg;
end


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function a=F_b_org(t,p);
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
a=p.F_ep0;


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function a=F_H2S_mixing_ell(t,p,H2S_T_ell);
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% moles of S per unit time diffusing up from the deep ocean into the
% photic aerobic zone.
if p.do_H2S_mixing_and_oxidation==1; 
  a=H2S_T_ell/p.tau_mixing;
else
  a=0; 
end


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function a=F_H2S_pyrite_ell(t,p,H2S_T_ell);
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% moles of S per unit time deposited as pyrite.
if p.do_pyrite_formation==1; 
  a=H2S_T_ell/p.tau_pyrite;
else
  a=0; 
end


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function a=F_Fe2p_mixing_ell(t,p,Fe2p);
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% moles of Fe2p per unit time diffusing up from the deep ocean into the
% photic aerobic zone.
if p.do_Fe2p_mixing_and_oxidation==1; 
  a=Fe2p/p.tau_mixing;
else
  a=0; 
end


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function a=F_Fe2p_siderite_ell(t,p,Fe2p);
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%% moles of Fe2p (or, equivalently, CO3^{2-}) per unit time deposited as 
%% sidrite (FeCO3) per unit time. 
if p.do_siderite_formation==1; 
  a=Fe2p/p.tau_siderite;
else
  a=0; 
end

    
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function a=temperature(t,p);
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Celsius!
a=p.Temperature_C_0;


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%% The following functions calculate the dissociation coefficients
%% needed for calculation of pCO2, and the solubility of oxygen,
%% taken from paleo_co2_tutorial.m, with slight naming changes.
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

function K0=K_0(p,P,T,S)
%     solubility of CO2 in seawater.  
%     by definition, CO2*=H2CO3*=CO2(aq)+H2CO3.
%         [CO2*]
%     K_0=-------- 
%         [CO2g]
%     from Zeebe's book
% Eli: looks like units are mole CO2/(kg Sea Water*atm)
% which is the same as the units used here: micro mole CO2/(kg SW*ppm)
T=T+273.15;
K0=exp( ...
    (9345.17/T)-60.2409+23.3585*log(T/100.) ...
    +S*( 0.023517-0.00023656*T  +  ...
         0.00000047036*T*T ));


function K1=K_1(p,P,T,S)
%        [H+][HCO3-]  
%     K_1=-----------      
%          [CO2(aq)]
%     from Millero (1987)

%% Old:
%% T=T+273.15;
%% K1=10.^(-((6320.18/T)-126.3405+19.568*log(T)+ ...
%% 	      (19.894-(840.39/T)-3.0189*log(T))* ...
%% 	      sqrt(S)+0.00678*S) );

%% From Zeebe's book, Eli:
T=T+273.15;
tmp1 = 2.83655 - 2307.1266 ./ T - 1.5529413 .* log(T);
tmp2 =         - (0.20760841 + 4.0484 ./ T) .* sqrt(S);
tmp3 =         + 0.08468345 .* S - 0.00654208 .* S .* sqrt(S);   
tmp4 =         + log(1 - 0.001005 .* S);
lnK1roy = tmp1 + tmp2 + tmp3 + tmp4;
K1 = exp(lnK1roy);

%% pressure effect:
K1 = K1*exp(p.lnkpok0(1));

% convert units to micromole:
K1=K1*1.e6;


function K2=K_2(p,P,T,S)
%               [H+][CO3--]                         
%        and K_2=-----------  from Millero (1987)
%                 [HCO3-]

T=T+273.15;
%% Old:
%K2=10.^(-((5143.69/T)-90.1833+14.613*log(T)+ ...
%	      (17.176-(690.59/T)-2.6719*log(T))* ...
%	      sqrt(S)+0.02169*S) );

%% Zeebe's book, Eli:
tmp1 = -9.226508 - 3351.6106 ./ T - 0.2005743 .* log(T);
tmp2 = (-0.106901773 - 23.9722 ./ T) .* sqrt(S);
tmp3 = 0.1130822 .* S - 0.00846934 .* S.^1.5 + log(1 - 0.001005 * S);
lnK2roy = tmp1 + tmp2 + tmp3;
K2 = exp(lnK2roy);

%% pressure effect:
K2 = K2*exp(p.lnkpok0(2));

% convert units to micromole:
K2=K2*1.e6;


function Kb=K_B(p,P,T,S)
%     for Boric acid,     [H+][B(OH4)-]
%                    K_B=-------------- is from Johansson & Wedborg (1982)
%                          [B(OH)3] 

T=T+273.15;
%% Old:
%output=10.^(-((1030.5/T)+5.5076-0.015469*S+ ...
%	      1.5339d-4*S*S ) );

%% Zeebe's book, Eli:
tmp1 =  (-8966.90-2890.53*sqrt(S)-77.942*S+1.728*S.^(3./2.)-0.0996*S.*S);
tmp2 =   +148.0248+137.1942*sqrt(S)+1.62142*S;
tmp3 = +(-24.4344-25.085*sqrt(S)-0.2474*S).*log(T);
lnKb = tmp1 ./ T + tmp2 + tmp3 + 0.053105*sqrt(S).*T;
Kb = exp(lnKb);

%% pressure effect:
Kb = Kb*exp(p.lnkpok0(3));

% convert units to micromole:
Kb=Kb*1.e6;


function Kw=K_w(p,P,T,S)
%     for water. K_w=[h+]*[OH-], from Dickson & Riley (1979)
%     pKw=3441.0*(1./T)+2.241-0.09415*sqrt(S)

%% Old:
%%T=T+273.15;
%%output=10.^(-((3441.0/T)+2.241-0.09415*sqrt(S) ));

%% Zeebe's book, Eli:
T=T+273.15;
Kw=exp(148.96502-13847.26/T-23.6521*log(T) ...
           +(118.67/T-5.977+1.0495*log(T))*sqrt(S)-0.01615*S);

%% pressure effect:
Kw = Kw*exp(p.lnkpok0(4));

% convert units to micromole:
Kw=Kw*1.e12;


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function K=K_sp(p,P,T,S)
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
  if p.do_use_Ksp_aragonite==1;
    K=K_sp_aragonite(p,P,T,S);
  else
    K=K_sp_calcite(p,P,T,S);
  end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function output=K_sp_calcite(p,P,T,S)
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%% The Calcite solubility product according to Mucci (1983) on the molal scale.
%% from Jack Silverman.
%P=p.K_sp_pressure; % atm
%dVcal=-(47.5-0.23.*T);
%Kcal=10.^(-171.9065-0.077993.*(T+273.15)+2839.319./(T+273.15)+...
%   71.595.*log10(T+273.15)+(-0.77712+0.0028426.*(T+273.15)+178.34./(T+...
%   273.15)).*S.^0.5-0.07711.*S+0.0041249.*S.^1.5);
%Kcalp=Kcal.*10.^-(dVcal.*(P-1)./(2.303*82.057834.*(273.15+T)));

%% Zeebe's book, Eli:
T=T+273.15;
tmp1 = -171.9065-0.077993.*T+2839.319./T+71.595.*log10(T);
tmp2 = +(-0.77712+0.0028426.*T+178.34./T).*sqrt(S);
tmp3 = -0.07711.*S+0.0041249.*S.^1.5;
log10Kspc = tmp1 + tmp2 + tmp3;
Kspc = 10.^(log10Kspc);

%% pressure effect:
Kspc = Kspc*exp(p.lnkpok0(7));

%% Eli: convert to micromole^2:
output=Kspc*1.e12;


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function Kspa=K_sp_aragonite(p,P,T,S)
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%% The agonite solubility product according to Mucci (1983) on the molal scale.
%% from Jack Silverman.
P=p.K_sp_pressure; % atm

%% Old:
% dVarg=-(45-0.23.*T);
% Kspa=10.^(-171.945-0.077993.*(T+273.15)+2903.293./(T+273.15)+...
%    71.595.*log10(T+273.15)+(-0.068393+0.0017276.*(T+273.15)+...
%    88.135./(T+273.15)).*S.^0.5-0.10018.*S+0.0059415.*S.^1.5);
% Kspa=Kspa.*10.^-(dVarg.*(P-1)./(2.303*82.057834.*(273.15+T)));

%% Zeebe's book, Eli:
T=T+273.15;
tmp1 = -171.945-0.077993.*T+2903.293./T+71.595.*log10(T);
tmp2 = +(-0.068393+0.0017276.*T+88.135./T).*sqrt(S);
tmp3 = -0.10018.*S+0.0059415.*S.^1.5;
log10Kspa = tmp1 + tmp2 + tmp3;
Kspa = 10.^(log10Kspa);

%% pressure effect:
Kspa = Kspa*exp(p.lnkpok0(8));

% Eli: convert to micromole^2:
Kspa=Kspa*1.e12;


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function output=K_HSO4(p,P,T,S)
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%% P in bar, T celsius, S ppt
%            [H+][SO4--]  
%     K_HSO4=-----------      
%              [HSO4-]
P=p.K_sp_pressure; % atm

if 1
  %% from aquaEnv: http://r-forge.r-project.org/projects/aquaenv/
  %% AquaEnv - Aquatic modelling Environment in R, Andreas by
  %% F. Hofmann, Karline Soetaert, Filip J. R. Meysman, 2009
  p=P;
  T         = T + 273.15;
  %% ionic strength in mol/kg-solution (molinity)
  %% salinity S in practical salinity units:
  I=(19.924*S/(1000-1.005*S));
  Iterms=[I,I^2,sqrt(I),I^1.5];
  A =  324.57*Iterms(3) - 771.54*Iterms(1) + 141.328;
  B =   35474*Iterms(1) +   1776*Iterms(2) - 13856*Iterms(3) - 2698*Iterms(4) - 4276.1;
  C = 114.723*Iterms(1) - 47.986*Iterms(3) - 23.093;
  D = 0;
  E = 0;
  lnK = A + (B/T) + C*log(T) + D*T + E*T^2;
  t         = T;
  t2        = t^2;
  coeff = [-18.03, 0.0466, 0.3160e-3,- 4.53, 0.0900,0];
  deltaV    = coeff(1) + coeff(2)*t + coeff(3)*t2;
  deltaK    = (coeff(4) + coeff(5)*t + coeff(6)*t2)/1000; % the 1000 is correction from Lewis1998
  R=83.14472;        % (bar*cm3)/(mol*K) the gas constant 
  deltaPlnK = -(deltaV/(R*T))*p + (0.5*(deltaK/(R*T)))*(p^2); 
  %% salinity S in practical salinity units the conversion factor from
  %% molality (mol/kg-H2O) to molinity (mol/kg-solution)
  molal2molin=(1-0.001005*S);
  K_HSO4 = exp(lnK + deltaPlnK + log(molal2molin));
  % conversion factor to pH_{NBS} scale (Pilson table F.2, p 405):
  f_H=0.739+0.0307*S+0.0000794*S^2+0.00006443*T-0.000117*S*T;
  output=K_HSO4*f_H;
  %% convert units to micromole:
  output=output*1.e6;
end

if 0
  %% Khoo 1977 from UNESCO report:
  T=T+273.15;
  lnT=log(T);
  lnBeta=1121.1/T-45.2167+7.484*lnT+0.0011984*S-1.2613e-4*S^2;
  beta=exp(lnBeta);
  output=1/beta;
end

if 0
  %% From "An introduction to the chemistry of the sea" by Michael
  %% E. Q. Pilson; based on Dickson 1990: j.chem.thermodynamics, vol 22,
  %% pp 113-127. "Standard potential of the reaction : AgCl(s) + H2(g)
  %% = Ag(s) + HCl(aq), and the standard acidity constant of the ion
  %% HSO4- in synthetic sea water from 273.15 to 318.15 K".
  T=T+273.15;
  lnT=log(T);
  I=19.92*S/(1000-1.00488*S);
  
  lnK_T=-4276.1/T+141.328-23.093*lnT+(-13856/T+324.57+47.986*lnT)*sqrt(I) ...
        +(35474/T-771.54+114.723*lnT)*I+(-2698/T)*I^(1.5)+(1776/T)*I^2;
  
  lnK_T
  output=exp(lnK_T);
end


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%function output=K0_H2S(p,P,T,S)
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%         [H2S(aq)]
%     K0=-------- 
%         [H2S(g)]
% Henry constant for H2S
% documentation: Doc/K0_H2S.tex
% output = 0.100908; % micromole/(kg*ppm atm);
  
  
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function output=K1_H2S(p,P,T,S)
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%% first dissociation constant for H2S
%% From AquaEnv, compared for P=0 to Table F.6 p 408, Pilson's book.
P=p.K_sp_pressure; % atm
T = T + 273.15;
sqrtS  = sqrt(S);
Sterms = [S^2, sqrtS, S*sqrtS];
A = 225.838  + 0.3449*Sterms(2) - 0.0274*S;
B = -13275.3;
C = -34.6435;
D = 0;
E = 0;
lnK = A + (B/T) + C*log(T) + D*T + E*T^2;
R=83.14472;        % (bar*cm3)/(mol*K) the gas constant 
t = T;
t2        = t^2;
coeff=[-14.80, 0.0020, -0.4000e-3,  2.89, 0.0540, 0];
deltaV    = coeff(1) + coeff(2)*t + coeff(3)*t2;
deltaK    = (coeff(4) + coeff(5)*t + coeff(6)*t2)/1000; %the 1000 is correction from Lewis1998
deltaPlnK = -(deltaV/(R*T))*P + (0.5*(deltaK/(R*T)))*(P^2); 

K_H2S = exp(lnK + deltaPlnK);
%% + log(scaleconvert(S, t, P=0, SumH2SO4, SumHF, khf=khf, khso4=khso4)$tot2sws) +
%% + log(scaleconvert(S, t, P, SumH2SO4, SumHF, khf=khf, khso4=khso4)$sws2free))

output=K_H2S;
%% conversion factor to pH_{NBS} scale (Pilson table F.2, p 405):
%f_H=0.739+0.0307*S+0.0000794*S^2+0.00006443*T-0.000117*S*T;
%output=output*f_H;

%% convert units to micromole:
output=output*1.e6;


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function [pH,Hp,OHm,CO2g,H2CO3,HCO3m,CO3m2,BOH3,BOH4m,HSO4m,SO4m2,H2Saq,HSm] ...
      =solve_carbonate_system_carbonate_only_iterative(t,p,Alk,C_T,B_T,SO4_T,H2S_T_ell,T,S,init_guess)
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
persistent n_error_messages

% Inputs:
% -------

% time (used only for printing messages), alkalinity, total CO2,
% temperature, salinity, pressure.

% Outputs:
% --------
% CO2g    = co_2(g)
% H2CO3    = H_2co_3^*  = H_2co_3+co_2(aq)
% OHm      = OH^-
% Hp       = H^+
% pH       = -log10(Hp)
% HCO3m    = HCO_3^-
% CO3m2    = CO_3^=
% BOH3   = B(OH)_3
% BOH4m  = B(OH)_4^-

% calculate dissolution coefficients to save operations during
% the iterations below:
P=p.K_sp_pressure; % atm
K0=K_0(p,P,T,S);
K1=K_1(p,P,T,S);
KB=K_B(p,P,T,S);
K2=K_2(p,P,T,S);
Kw=K_w(p,P,T,S);
% initialize a=1/[H^+] and ALK_C from previous time step:
a=1/init_guess(1);
max_carbonate_iterates=50;
min_carbonate_iterates=5;
if isempty(n_error_messages); n_error_messages=0; end
iterate=1;
epsilon_a=1.0;
while iterate<min_carbonate_iterates || ( iterate<max_carbonate_iterates && epsilon_a > 1.0e-6 )
  iterate=iterate+1;
  %% Calculate carbonate alkalinity:
  ALK_C=Alk-( (a*KB*B_T)/(1+a*KB) + a*Kw - 1/a );
  %% gamma:
  gamma=ALK_C/C_T;
  %% solve for a=1/[H^+]:
  a_new=(-(1-gamma)*K1...
         +sqrt((1-gamma)^2*K1^2 ...
               + 4*(2-gamma)*K1*K2*gamma)...
         )/(2*(2-gamma)*K1*K2);
  %% evaluate relative increment to a in this iteration:
  epsilon_a=abs(a_new-a)/a;
  a=a_new;
end
if iterate == max_carbonate_iterates && n_error_messages<6
  n_error_messages=n_error_messages+1;
  fprintf(1,['reached max carbonate system iterates,' ...
             ' t=%g, iterate=%d, Alk=%8.2f, C_T=%8.2f\n'],t,iterate,Alk,C_T);
 elseif n_error_messages==6
  n_error_messages=n_error_messages+1;
   fprintf(1,'not printing any more maximum number of iterates warnings.\n');
end


%% Now solve for all other carbonate system variables:
Hp       = 1/a;
% add 6 to pH because units of Hp are micro mole/liter:
pH       = -log10(Hp)+6;  
OHm      = Kw*a;
H2CO3    = (1/(a*K1+a^2*K1*K2+1))*C_T;
HCO3m    = ((a*K1)/(a*K1+a^2*K1*K2+1))*C_T;
CO3m2    = ((a^2*K1*K2)...
            /(a*K1+a^2*K1*K2+1))*C_T;
CO2g    = H2CO3/K0;
BOH4m  = ((a*KB)/(1+a*KB))*B_T;
BOH3   = BOH4m/(a*KB);
HSO4m=NaN;
SO4m2=NaN;
H2Saq=NaN;
HSm=NaN;


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function [pH,Hp,OHm,CO2g,H2CO3,HCO3m,CO3m2,BOH3,BOH4m,HSO4m,SO4m2,H2Saq,HSm] ...
    =solve_carbonate_system_approx(t,p,Alk,C_T,B_T,SO4_T,H2S_T_ell,T,S,init_guess)
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

P=1;
HCO3m=2*C_T-Alk;
CO3m2=Alk-C_T;
Hp=K_2(p,P,T,S)*(2*C_T-Alk)/(Alk-C_T);
H2CO3=(K_2(p,P,T,S)/K_1(p,P,T,S))*(2*C_T-Alk)^2/(Alk-C_T);
CO2g=H2CO3/K_0(p,P,T,S);
pH=-log10(Hp)+6;
OHm=NaN;
HCO3m=NaN;
CO3m2=NaN;
BOH3=NaN;
BOH4m=NaN;
HSO4m=NaN;
SO4m2=NaN;
H2Saq=NaN;
HSm=NaN;

  
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function [pH,Hp,OHm,CO2g,H2CO3,HCO3m,CO3m2,BOH3,BOH4m,HSO4m,SO4m2,H2Saq,HSm] ...
  =solve_carbonate_system_with_sulfur(t,p,Alk,C_T,B_T,SO4_T,H2S_T_ell,T,S,init_guess)
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

p_fsolve.C_T=C_T;
p_fsolve.B_T=B_T;
p_fsolve.SO4_T=SO4_T;
p_fsolve.H2S_T_ell=H2S_T_ell;
p_fsolve.Alk=Alk;
p_fsolve.T=T;
p_fsolve.S=S;

% misc reaction rate constants that need to be calculated from T,S,P
% only once per time step, so are better done here than in
% carbonate_sulfate_eqns:
P=p.K_sp_pressure;
T=temperature(t,p);
p=pressure_effect_on_K(t,p);
S=p.Salinity0;
K.K_w=K_w(p,P,T,S);
K.K_0=K_0(p,P,T,S);
K.K_1=K_1(p,P,T,S);
K.K_2=K_2(p,P,T,S);
K.K_HSO4=K_HSO4(p,P,T,S);
K.K_B=K_B(p,P,T,S);
K.K1_H2S=K1_H2S(p,P,T,S);

%% nondimensionalize initial guess:
init_guess=(init_guess-p.carb_sulf_offset)./p.carb_sulf_scale;
[x,fval,exitflag,output,J]=fsolve(@(X)carbonate_sulfate_eqns(X,t,p,p_fsolve,K),init_guess,p.optim_options);
%% dimensionalize the fsolve solution:
X=x.*p.carb_sulf_scale+p.carb_sulf_offset;

%fprintf(1,'iterations=%d\n',output.iterations);

if 0 %% for debugging:
  format long
  disp('in solve_carbonate_system_with_sulfur:, nondim x=');
  disp(x);
  disp('dimensional X=');
  disp(X);
end

if exitflag<1
  output
  output.message
  exitflag
  fval
  fprintf(1,'t=%gyrs\n',t);
  fprintf(1,'F='); fprintf(1,'%g; ',fval); fprintf(1,'\n');
  fprintf(1,'X='); fprintf(1,'%g; ',X); fprintf(1,'\n');
  t,p,Alk,C_T,B_T,SO4_T,H2S_T_ell,T,S,init_guess
  disp('*** fsolve problem, pausing in solve_carbonate_system_with_sulfur.');
  pause
end

% assign output arguments:
Hp      = X(1);
OHm     = X(2);
CO2g    = X(3);
H2CO3   = X(4);
HCO3m   = X(5);
CO3m2   = X(6);
BOH3    = X(7);
BOH4m   = X(8);
HSO4m   = X(9);
SO4m2   = X(10);
H2Saq   = X(11);
HSm     = X(12);
pH=-log10(Hp)+6;

if 0
  fprintf(1,'solution:\n')
  for i=1:length(X)
    lab=p.carb_sulf_label(i,:);
    fprintf(1,'%s=%g,\n',lab,X(i));
  end
  fprintf(1,'pH   =%g\n',-log10(X(1))+6);
  fprintf(1,'\n')
end
%fprintf(1,'J at end of optimization:\n'); printJ(J);


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function [F,J]=carbonate_sulfate_eqns(X,t,p,p_fsolve,K)
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Called by fsolve which is invoked in solve_carbonate_sulfate_system;
% returns the equations of the carbonate+sulfate system and the
% Jacobian.

  %% get variables from unknown vector X:  X is nondimensional, so
  %% for example X(1)=(Hp-p.carb_sulf_offset(1))/p.carb_sulf_scale(1):
  Hp      = X(1) *p.carb_sulf_scale(1) +p.carb_sulf_offset(1);
  OHm     = X(2) *p.carb_sulf_scale(2) +p.carb_sulf_offset(2);
  CO2g    = X(3) *p.carb_sulf_scale(3) +p.carb_sulf_offset(3);
  H2CO3   = X(4) *p.carb_sulf_scale(4) +p.carb_sulf_offset(4);
  HCO3m   = X(5) *p.carb_sulf_scale(5) +p.carb_sulf_offset(5);
  CO3m2   = X(6) *p.carb_sulf_scale(6) +p.carb_sulf_offset(6);
  BOH3    = X(7) *p.carb_sulf_scale(7) +p.carb_sulf_offset(7);
  BOH4m   = X(8) *p.carb_sulf_scale(8) +p.carb_sulf_offset(8);
  HSO4m   = X(9) *p.carb_sulf_scale(9) +p.carb_sulf_offset(9);
  SO4m2   = X(10)*p.carb_sulf_scale(10)+p.carb_sulf_offset(10);
  H2Saq   = X(11)*p.carb_sulf_scale(11)+p.carb_sulf_offset(11);
  HSm     = X(12)*p.carb_sulf_scale(12)+p.carb_sulf_offset(12);

  %% get some additional needed quantities:
  C_T   = p_fsolve.C_T;
  B_T   = p_fsolve.B_T;
  SO4_T   = p_fsolve.SO4_T;
  H2S_T_ell = p_fsolve.H2S_T_ell;
  Alk   = p_fsolve.Alk;

  %% Calculate equations:
  F=[ ...
    K.K_w-Hp*OHm,
    K.K_0*CO2g-H2CO3,
    K.K_1*H2CO3-Hp*HCO3m,
    K.K_2*HCO3m-Hp*CO3m2,
    K.K_HSO4*HSO4m-Hp*SO4m2,
    K.K_B*BOH3-Hp*BOH4m,
    K.K1_H2S*H2Saq-Hp*HSm,
    H2CO3+HCO3m+CO3m2-C_T,
    BOH4m+BOH3-B_T,
    HSO4m+SO4m2-SO4_T,
    HSm+H2Saq-H2S_T_ell,
    HCO3m+2*CO3m2+BOH4m+HSO4m+2*SO4m2+HSm+OHm-Hp-Alk ...
    ]';
  
  %% Calculate Jacobian:
  
  J=[ -OHm,	-Hp,	0,	0,	0,	0,	0,	0,	0,		0,	0,		0 ;
      0,	0,	K.K_0,	-1,	0,	0,	0,	0,	0,		0,	0,		0 ;
      -HCO3m,	0,	0,	K.K_1,	-Hp,	0,	0,	0,	0,		0,	0,		0 ;
      -CO3m2,	0,	0,	0,	K.K_2,	-Hp,	0,	0,	0,		0,	0,		0 ;
      -SO4m2,	0,	0,	0,	0,	0,	0,	0,	K.K_HSO4,	-Hp,	0,		0 ;
      -BOH4m,	0,	0,	0,	0,	0,	K.K_B,	-Hp,	0,		0,	0,		0 ;
      -HSm,	0,	0,	0,	0,	0,	0,	0,	0,		0,	K.K1_H2S,	-Hp ;
      0,	0,	0,	1,	1,	1,	0,	0,	0,		0,	0,		0 ;
      0,	0,	0,	0,	0,	0,	1,	1,	0,		0,	0,		0 ;
      0,	0,	0,	0,	0,	0,	0,	0,	1,		1,	0,		0 ;
      0,	0,	0,	0,	0,	0,	0,	0,	0,		0,	1,		1 ;
      -1,	1,	0,	0,	1,	2,	0,	1,	1,		2,	0,		1 ];

  if 0
    %% Add inequality penalties in case the solver gets to negative
    %% carbonate variables:
    %% dimensional variables:
    Xdim=X.*p.carb_sulf_scale+p.carb_sulf_offset;
    %% find indices of negative variables:
    I_penalty=find(Xdim<0);
    %% add penalty:
    F(I_penalty)=F(I_penalty)+Xdim(I_penalty).^2;
    %% add to Jacobian:
    for i=I_penalty
      J(i,i)=J(i,i)+2*Xdim(i);
    end
  end
  
  %% scale the Jacobian:
  N=length(F);
  for i=1:N
    J(i,:)=J(i,:).*p.carb_sulf_scale(:)';
  end

%%fprintf(1,'analytical J:\n'); printJ(J); pause

%fprintf(1,'F='); fprintf(1,'%g; ',F); fprintf(1,'\n');


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function printJ(J)
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
N=length(J(:,1));
for i=1:N
  fprintf(1,'%g, ',J(i,:)); fprintf(1,'\n');
end


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function [pH,Hp,OHm,CO2g,H2CO3,HCO3m,CO3m2,BOH3,BOH4m,HSO4m,SO4m2,H2Saq,HSm,T] ...
    =calc_pCO2(t,p,C_T,B_T,SO4_T,H2S_T_ell,Alk,Ca2p)
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%% solve the carbonate system to calculate pCO2.  
%% C_T: a timeseries, in units of micromol/kg.

  N=length(C_T);
  Hp=C_T*NaN;
  CO2g=C_T*NaN;
  CO3m2=C_T*NaN;
  HCO3m=C_T*NaN;
  H2CO3=C_T*NaN;
  OHm=C_T*NaN;
  BOH3=C_T*NaN;
  BOH4m=C_T*NaN;
  HSO4m=C_T*NaN;
  SO4m2=C_T*NaN;
  HSm=C_T*NaN;
  H2Saq=C_T*NaN;
  T=C_T*NaN;
  S=p.Salinity0;
  for i=1:N
    T(i)=temperature(t(i),p);
    if i>1; 
      init_guess=[Hp_t,OHm_t,CO2g_t,H2CO3_t,HCO3m_t,CO3m2_t,BOH3_t,BOH4m_t,HSO4m_t,SO4m2_t,H2Saq_t,HSm_t]; 
    else; 
      init_guess=p.carb_sulf_init_guess; 
    end
    if strcmp(p.carbonate_system_solver,'standard')==1
      [pH_t,Hp_t,OHm_t,CO2g_t,H2CO3_t,HCO3m_t,CO3m2_t,BOH3_t,BOH4m_t,HSO4m_t,SO4m2_t,H2Saq_t,HSm_t] ...
        =solve_carbonate_system_carbonate_only_iterative(t(i),p,Alk(i),C_T(i),B_T(i),SO4_T(i),H2S_T_ell(i),T(i),S,init_guess);
    elseif strcmp(p.carbonate_system_solver,'sulfur')==1
      [pH_t,Hp_t,OHm_t,CO2g_t,H2CO3_t,HCO3m_t,CO3m2_t,BOH3_t,BOH4m_t,HSO4m_t,SO4m2_t,H2Saq_t,HSm_t] ...
        =solve_carbonate_system_with_sulfur(t(i),p,Alk(i),C_T(i),B_T(i),SO4_T(i),H2S_T_ell(i),T(i),S,init_guess);
    elseif strcmp(p.carbonate_system_solver,'approx')==1
      [pH_t,Hp_t,OHm_t,CO2g_t,H2CO3_t,HCO3m_t,CO3m2_t,BOH3_t,BOH4m_t,HSO4m_t,SO4m2_t,H2Saq_t,HSm_t] ...
        =solve_carbonate_system_approx(t(i),p,Alk(i),C_T(i),B_T(i),SO4_T(i),H2S_T_ell(i),T(i),S,init_guess);
    end
    CO2g(i)=CO2g_t;
    Hp(i)=Hp_t;
    CO3m2(i)=CO3m2_t;
    HCO3m(i)=HCO3m_t;
    H2CO3(i)=H2CO3_t;
    OHm(i)=OHm_t;
    BOH3(i)=BOH3_t;
    BOH4m(i)=BOH4m_t;
    HSO4m(i)=HSO4m_t;
    SO4m2(i)=SO4m2_t;
    HSm(i)=HSm_t;
    H2Saq(i)=H2Saq_t;
    pH(i)=-log10(Hp(i))+6;
  end

  if min(Hp)<0
    fprintf('*** negative min(Hp)=%g\n',min(Hp));
    pause
  end
  if min(CO2g)<0
    fprintf('*** negative min(CO2g)=%g\n',min(CO2g));
    pause
  end


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function test_K_sp_and_friends(t,p)
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% used for debugging the K_sp routines for calcite and aragonite

T=temperature(t,p); S=35; P=0;
p.K_sp_pressure=0; P1=p.K_sp_pressure; % atm
p=pressure_effect_on_K(t,p);
K0a=-log10(K_0(p,P,T,S));
K1a=-log10(K_1(p,P,T,S));
KBa=-log10(K_B(p,P,T,S));
K2a=-log10(K_2(p,P,T,S));
Kwa=-log10(K_w(p,P,T,S));
Kspa=-log10(K_sp(p,P,T,S));
p.K_sp_pressure=400; P2=p.K_sp_pressure; % atm
p=pressure_effect_on_K(t,p);
K0b=-log10(K_0(p,P,T,S));
K1b=-log10(K_1(p,P,T,S));
KBb=-log10(K_B(p,P,T,S));
K2b=-log10(K_2(p,P,T,S));
Kwb=-log10(K_w(p,P,T,S));
Kspb=-log10(K_sp(p,P,T,S));

fprintf(1,' pK		P=%11.8g bars	 P=%11.8g bars\n',P1,P2);
fprintf(1,'pK0		%11.8g		 %11.8g\n',K0a,K0b);
fprintf(1,'pK1		%11.8g		 %11.8g\n',K1a,K1b);
fprintf(1,'pK2		%11.8g		 %11.8g\n',K2a,K2b);
fprintf(1,'pKB		%11.8g		 %11.8g\n',KBa,KBb);
fprintf(1,'pKw		%11.8g		 %11.8g\n',Kwa,Kwb);
fprintf(1,'pKsp		%11.8g		 %11.8g\n',Kspa,Kspb);
disp('pausing...');
pause

figure(1); clf;
T=0:0.1:40; S=35;  p.K_sp_pressure=400; P=p.K_sp_pressure;
KSP_aragonite=T*NaN;  KSP_calcite=T*NaN;
for i=1:length(T);
  %% This needs to be called whenever the temperature or pressure change:
  p=pressure_effect_on_K(t,p);
  KSP_aragonite(i)=K_sp_aragonite(p,P,T(i),S); 
  KSP_calcite(i)=K_sp(p,P,T(i),S); 
end

subplot(2,1,1)
plot(T,KSP_aragonite,'g'); 
hold on; 
plot(T,KSP_calcite,'r');
title('KSP(T)')
h1=title(sprintf('concentrations in \\mu mole, S=%g ppt, P=%g atm',S,p.K_sp_pressure));
h2=xlabel('T');
h3=ylabel('K_{sp}');
h4=legend('aragonite','calcite');
set([h1,h2,h3,h4,gca],'fontsize',12);

P=0:1:600; T=4;
KSP_aragonite=P*NaN;
KSP_calcite=P*NaN;
for i=1:length(P);
  p.K_sp_pressure=P(i);
  p=pressure_effect_on_K(t,p);
  KSP_aragonite(i)=K_sp_aragonite(p,P(i),T,S); 
  KSP_calcite(i)=K_sp(p,P(i),T,S); 
end

subplot(2,1,2)
plot(P,KSP_aragonite,'g'); 
hold on; 
plot(P,KSP_calcite,'r');
title('KSP(P)')
h1=title(sprintf('concentrations in \\mu mole, S=%g ppt, T=%g C',S,T));
h2=xlabel('P');
h3=ylabel('K_{sp}');
h4=legend('aragonite','calcite');
set([h1,h2,h3,h4,gca],'fontsize',12);

disp('done plotting Ksp.')


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function calc_simple_scenarios(p)
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
  t=0;
  T=25;
  S=35;
  init_guess=p.carb_sulf_init_guess;
  n=0; fprintf(1,'running scenarios: ');
  
  %% control
  n=n+1; fprintf(1,'%d, ',n);
  a_title(n,:)='control  ';
  C_T(n)=2040;SO4_T(n)=0;B_T(n)=400;H2S_T_ell(n)=0; Alk(n)=2300;
  [pH(n),Hp(n),OHm(n),CO2g(n),H2CO3(n),HCO3m(n),CO3m2(n),BOH3(n),BOH4m(n),HSO4m(n),SO4m2(n),H2Saq(n),HSm(n)] ...
    =solve_carbonate_system_with_sulfur(t,p,Alk(n),C_T(n),B_T(n),SO4_T(n),H2S_T_ell(n),T,S,init_guess);
  
  %% control with sulfur
  n=n+1; fprintf(1,'%d, ',n);
  a_title(n,:)='ctrl w/S ';
  C_T(n)=2040;SO4_T(n)=28000;B_T(n)=400;H2S_T_ell(n)=100; Alk(n)=2300+2*28000;
  [pH(n),Hp(n),OHm(n),CO2g(n),H2CO3(n),HCO3m(n),CO3m2(n),BOH3(n),BOH4m(n),HSO4m(n),SO4m2(n),H2Saq(n),HSm(n)] ...
    =solve_carbonate_system_with_sulfur(t,p,Alk(n),C_T(n),B_T(n),SO4_T(n),H2S_T_ell(n),T,S,init_guess);
  
  %% standard remineralization, no Sulfur
  n=n+1; fprintf(1,'%d, ',n);
  a_title(n,:)='std remin';
  C_T(n)=2240;SO4_T(n)=0;B_T(n)=400;H2S_T_ell(n)=0; Alk(n)=2300;
  [pH(n),Hp(n),OHm(n),CO2g(n),H2CO3(n),HCO3m(n),CO3m2(n),BOH3(n),BOH4m(n),HSO4m(n),SO4m2(n),H2Saq(n),HSm(n)] ...
    =solve_carbonate_system_with_sulfur(t,p,Alk(n),C_T(n),B_T(n),SO4_T(n),H2S_T_ell(n),T,S,init_guess);
  
  %% standard remineralization with sulfur
  n=n+1; fprintf(1,'%d, ',n);
  a_title(n,:)='std rem+S';
  C_T(n)=2240;SO4_T(n)=28000;B_T(n)=400;H2S_T_ell(n)=100; Alk(n)=2300+2*28000;
  [pH(n),Hp(n),OHm(n),CO2g(n),H2CO3(n),HCO3m(n),CO3m2(n),BOH3(n),BOH4m(n),HSO4m(n),SO4m2(n),H2Saq(n),HSm(n)] ...
    =solve_carbonate_system_with_sulfur(t,p,Alk(n),C_T(n),B_T(n),SO4_T(n),H2S_T_ell(n),T,S,init_guess);
  
  %% sulfate reduction
  n=n+1; fprintf(1,'%d, ',n);
  a_title(n,:)='sulfate  ';
  C_T(n)=2240;SO4_T(n)=27900;B_T(n)=400;H2S_T_ell(n)=200; Alk(n)=2300+2*28000;
  [pH(n),Hp(n),OHm(n),CO2g(n),H2CO3(n),HCO3m(n),CO3m2(n),BOH3(n),BOH4m(n),HSO4m(n),SO4m2(n),H2Saq(n),HSm(n)] ...
    =solve_carbonate_system_with_sulfur(t,p,Alk(n),C_T(n),B_T(n),SO4_T(n),H2S_T_ell(n),T,S,init_guess);
  
  fprintf(1,'  done. \n');
  table_filename='table1';
  print_table_of_simple_scenarios(p,n,a_title,table_filename ...
                                  ,C_T,SO4_T,B_T,H2S_T_ell,Alk,pH,Hp, ...
                                  OHm,CO2g,H2CO3,HCO3m,CO3m2,BOH3, ...
                                  BOH4m,HSO4m,SO4m2,H2Saq,HSm)
  
  %% ----------------------------------------------------------------------
  %% second set of experiments:
  %% ----------------------------------------------------------------------
  
  n=0; fprintf(1,'running scenarios: ');
  %% control, total H2S=100:
  n=n+1; fprintf(1,'%d, ',n);
  a_title(n,:)='ctr100H2S';
  C_T(n)=2040;SO4_T(n)=28000;B_T(n)=400;H2S_T_ell(n)=100; Alk(n)=2300+2*28000;
  [pH(n),Hp(n),OHm(n),CO2g(n),H2CO3(n),HCO3m(n),CO3m2(n),BOH3(n),BOH4m(n),HSO4m(n),SO4m2(n),H2Saq(n),HSm(n)] ...
    =solve_carbonate_system_with_sulfur(t,p,Alk(n),C_T(n),B_T(n),SO4_T(n),H2S_T_ell(n),T,S,init_guess);

  %% production of one mole of organic carbon and sulfate-reduction
  %% remineralization of two:
  n=n+1; fprintf(1,'%d, ',n);
  a_title(n,:)='slf100H2S';
  C_T(n)=2140;SO4_T(n)=27900;B_T(n)=400;H2S_T_ell(n)=200; Alk(n)=2300+2*28000;
  [pH(n),Hp(n),OHm(n),CO2g(n),H2CO3(n),HCO3m(n),CO3m2(n),BOH3(n),BOH4m(n),HSO4m(n),SO4m2(n),H2Saq(n),HSm(n)] ...
    =solve_carbonate_system_with_sulfur(t,p,Alk(n),C_T(n),B_T(n),SO4_T(n),H2S_T_ell(n),T,S,init_guess);

  fprintf(1,'  done. \n');

  table_filename='table2';
  print_table_of_simple_scenarios(p,n,a_title,table_filename ...
                                  ,C_T,SO4_T,B_T,H2S_T_ell,Alk,pH,Hp, ...
                                  OHm,CO2g,H2CO3,HCO3m,CO3m2,BOH3, ...
                                  BOH4m,HSO4m,SO4m2,H2Saq,HSm)

  %% ----------------------------------------------------------------------
  %% third set of experiments: CO2(g) as function of remineralization
  %% ----------------------------------------------------------------------

  n=0;
  for U=-100:10:3000
    %% sulfate reduction
    n=n+1;
    UU(n)=U;
    %% exact solution:
    C_T(n)=2040+2*U;SO4_T(n)=28000-U;B_T(n)=400;H2S_T_ell(n)=100+U; Alk(n)=2300+2*28000;
    [pH(n),Hp(n),OHm(n),CO2g(n),H2CO3(n),HCO3m(n),CO3m2(n),BOH3(n),BOH4m(n),HSO4m(n),SO4m2(n),H2Saq(n),HSm(n)] ...
      =solve_carbonate_system_with_sulfur(t,p,Alk(n),C_T(n),B_T(n),SO4_T(n),H2S_T_ell(n),T,S,init_guess);
    
    % approximate solution:
    Alk_SO4_analytic(n)=2300+2*U;
    Alk_O2_analytic(n)=2300;
    P=1;
    CO2g_SO4_analytic(n)=(K_2(p,P,T,S)/(K_1(p,P,T,S)*K_0(p,P,T,S))) ...
        *(2*C_T(n)-Alk_SO4_analytic(n)+H2S_T_ell(n))^2/(Alk_SO4_analytic(n)-C_T(n)-H2S_T_ell(n));
    CO2g_O2_analytic(n)=(K_2(p,P,T,S)/(K_1(p,P,T,S)*K_0(p,P,T,S))) ...
        *(2*C_T(n)-Alk_O2_analytic(n))^2/(Alk_O2_analytic(n)-C_T(n));
    Hp_SO4_analytic(n)=K_2(p,P,T,S) ...
        *(2*C_T(n)-Alk_SO4_analytic(n)+H2S_T_ell(n))/(Alk_SO4_analytic(n)-C_T(n)-H2S_T_ell(n));
    Hp_O2_analytic(n)=K_2(p,P,T,S) ...
        *(2*C_T(n)-Alk_O2_analytic(n))/(Alk_O2_analytic(n)-C_T(n));
    pH_SO4_analytic(n)=-log10(Hp_SO4_analytic(n))+6;
    pH_O2_analytic(n)=-log10(Hp_O2_analytic(n))+6;
  end
  figure(1);
  clf
  isubplot=0;
  nrow=2; ncol=2;

  isubplot=isubplot+1;
  subplot(nrow,ncol,isubplot)
  plot(UU,CO2g,'r');
  hold on
  plot(UU,CO2g_SO4_analytic,'g');
  plot(UU,CO2g_O2_analytic,'b');
  legend('SO4 exact','SO4 approx','O2 approx');
  h1=xlabel('U');
  h2=ylabel('CO2(g)');
  h3=title('remin scenarios');
  set([gca,h1,h2,h3],'fontsize',14);

  isubplot=isubplot+1;
  subplot(nrow,ncol,isubplot)
  plot(UU,pH,'r');
  hold on
  plot(UU,pH_SO4_analytic,'g');
  plot(UU,pH_O2_analytic,'b');
  legend('SO4 exact','SO4 approx','O2 approx');
  h1=xlabel('U');
  h2=ylabel('CO2(g)');
  h3=title('remin scenarios');
  set([gca,h1,h2,h3],'fontsize',14);
  
  disp('pausing...')
  pause
 
  
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function run_analytic_approximation_scenarios(p)
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
syms CO2g CT Alk H2ST U gamma %K2 K1 KH 

T=25; P=1; S=35;
KH=K_0(p,P,T,S);
K1=K_1(p,P,T,S);
K2=K_2(p,P,T,S);

gamma=1.2;

CT=2040+U*(2-gamma);
H2ST=200+U;
Alk=2300+200+2*U;
CO2g=(K2/(K1*KH))*((2*CT-Alk+H2ST)^2)/(Alk-CT-H2ST);

CO2g_control=subs((K2/(K1*KH))*((2*CT-Alk+H2ST)^2)/(Alk-CT-H2ST),U,0)
CO2g_scenario=subs((K2/(K1*KH))*((2*CT-Alk+H2ST)^2)/(Alk-CT-H2ST),U,100)

disp('pausing');
pause


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%    
function print_table_of_simple_scenarios(p,n,a_title,table_filename ...
                                  ,C_T,SO4_T,B_T,H2S_T_ell,Alk,pH,Hp, ...
                                  OHm,CO2g,H2CO3,HCO3m,CO3m2,BOH3, ...
                                  BOH4m,HSO4m,SO4m2,H2Saq,HSm)
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

  fid=fopen(sprintf('Output/%s.tex',table_filename),'w');

  Alk_C=HCO3m+2*CO3m2;
  DAlk_T=Alk; 
  for i=1:n; if SO4_T(i)~=0; DAlk_T(i)=Alk(i)-2*p.SO40; end; end
  
  fprintf(fid,'\n variable '); 
  for i=1:n; fprintf(fid,' & %s ',a_title(i,:)); end;  
  fprintf(fid,' \\\\ \n');
  fprintf(fid,'\\hline \n');
  fprintf(fid,'$H$'); fprintf(fid,' & %7.4g',pH(1:n)); fprintf(fid,' \\\\ \n');
  fprintf(fid,'$H^+$'); fprintf(fid,' & %7.4g',Hp(1:n)); fprintf(fid,' \\\\ \n');
  fprintf(fid,'$OH^-$'); fprintf(fid,' & %7.4g',OHm(1:n)); fprintf(fid,' \\\\ \n');
  fprintf(fid,'$CO_2(g)$'); fprintf(fid,' & %7.4g',CO2g(1:n)); fprintf(fid,' \\\\ \n');
  fprintf(fid,'$H_2CO_3^*$'); fprintf(fid,' & %7.4g',H2CO3(1:n)); fprintf(fid,' \\\\ \n');
  fprintf(fid,'$HCO_3^-$'); fprintf(fid,' & %7.4g',HCO3m(1:n)); fprintf(fid,' \\\\ \n');
  fprintf(fid,'$CO_3^{2-}$'); fprintf(fid,' & %7.4g',CO3m2(1:n)); fprintf(fid,' \\\\ \n');
  fprintf(fid,'$B(OH)_3$'); fprintf(fid,' & %7.4g',BOH3(1:n)); fprintf(fid,' \\\\ \n');
  fprintf(fid,'$B(OH)_4^-$'); fprintf(fid,' & %7.4g',BOH4m(1:n)); fprintf(fid,' \\\\ \n');
  fprintf(fid,'$HSO_4^-$'); fprintf(fid,' & %7.4g',HSO4m(1:n)); fprintf(fid,' \\\\ \n');
  fprintf(fid,'$SO_4^{2-}$'); fprintf(fid,' & %7.4g',SO4m2(1:n)); fprintf(fid,' \\\\ \n');
  fprintf(fid,'$H_2S(aq)$'); fprintf(fid,' & %7.4g',H2Saq(1:n)); fprintf(fid,' \\\\ \n');
  fprintf(fid,'$HS^-$'); fprintf(fid,' & %7.4g',HSm(1:n)); fprintf(fid,' \\\\ \n');
  fprintf(fid,'${\\Delta}Alk_T$'); fprintf(fid,' & %7.4g',DAlk_T(1:n)); fprintf(fid,' \\\\ \n');
  fprintf(fid,'$Alk_C$'); fprintf(fid,' & %7.4g',Alk_C(1:n)); fprintf(fid,' \\\\ \n');
  fprintf(fid,'$C_T$'); fprintf(fid,' & %7.4g',C_T(1:n)); fprintf(fid,' \\\\ \n');
  fprintf(fid,'$B_T$'); fprintf(fid,' & %7.4g',B_T(1:n)); fprintf(fid,' \\\\ \n');
  fprintf(fid,'$(SO_4)_T$'); fprintf(fid,' & %7.4g',SO4_T(1:n)); fprintf(fid,' \\\\ \n');
  fprintf(fid,'$(H_2S)_T$'); fprintf(fid,' & %7.4g',H2S_T_ell(1:n)); fprintf(fid,' \\\\ \n');
  fclose(fid);
  

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function test_carbonate_system(p)
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%% reproduce figure 1.1.3, p 7, from Zeebe and Wolf-Gladrow
%% or firgure 4-5 from Stum & Morgan, p 133
  t=0;
  T=temperature(t,p); % cannot change temp here, must change the
                      % temperature function.
  S=35;
  init_guess=p.carb_sulf_init_guess;
  P=500;
  p.K_sp_pressure=P;
  p=pressure_effect_on_K(t,p);
  %% first, print carbonate solution for a specific choice of the
  %% input parameters, to compare with Zeebe's csys3 matlab program:
  C_T=2000;
  Alk=2150+p.SO40*2;
  B_T=416;
  SO4_T=p.SO40;
  H2S_T_ell=0;
  disp('carbonate system solution:');
  [pH,Hp,OHm,CO2g,H2CO3,HCO3m,CO3m2,BOH3,BOH4m,HSO4m,SO4m2,H2Saq,HSm] ...
    =solve_carbonate_system_with_sulfur(t,p,Alk,C_T,B_T,SO4_T,H2S_T_ell,T,S,init_guess);
%    =solve_carbonate_system_carbonate_only_iterative(t,p,Alk,C_T,B_T,SO4_T,H2S_T_ell,T,S,init_guess);
  fprintf(1,[' t=%gMyr,\n' ...
             ' Alk=%g,\n C_T=%g,\n B_T=%g,\n SO4_T=%g,\n H2S_T_ell=%g,\n ' ...
             'CO2g=%g,\n H2CO3=%g,\n OHm=%g,\n Hp=%g,\n pH=%g,\n ' ...
             'HCO3m=%g,\n CO3m2=%g,\n BOH3=%g,\n BOH4m=%g\n'] ...
          ,t/1.e6,Alk,C_T,B_T,SO4_T,H2S_T_ell,CO2g,H2CO3,OHm,Hp,pH, ...
          HCO3m,CO3m2,BOH3,BOH4m);  

  CO3m2_sat=K_sp(p,P,T,S)/p.Ca2p0;
  fprintf(' CO3m2(sat)=%11.8g\n',CO3m2_sat);
  disp('pausing...');
  pause

  %% set input values for contour plot:  
  B_T=p.B_T0;
  SO4_T=p.SO40;
  H2S_T_ell=p.H2S0;
  
  %% contour:
  %% Zeebe:
  DIC_axis=1950:10:2150;
  Alk_axis=2250:10:2450;
  %% Stum & Morgan:
  %DIC_axis=0:100:2500;
  %Alk_axis=-500:100:3500;

  N_DIC=length(DIC_axis);
  N_Alk=length(Alk_axis);
  CO2g_contour=zeros(N_DIC,N_Alk);
  H2CO3_contour=zeros(N_DIC,N_Alk);
  pH_contour=zeros(N_DIC,N_Alk);
  fprintf(1,'working ');
  for i=1:N_DIC
    fprintf(1,'.');
    for j=1:N_Alk
      C_T=DIC_axis(i);
      Alk=Alk_axis(j);
      [pH,Hp,OHm,CO2g,H2CO3,HCO3m,CO3m2,BOH3,BOH4m,HSO4m,SO4m2,H2Saq,HSm] ...
        =solve_carbonate_system_carbonate_only_iterative(t,p,Alk,C_T,B_T,SO4_T,H2S_T_ell,T,S,init_guess);
%        =solve_carbonate_system_approx(t,p,Alk,C_T,B_T,SO4_T,H2S_T_ell,T,S,init_guess);
      H2CO3_contour(i,j)=H2CO3;
      CO2g_contour(i,j)=CO2g;
      pH_contour(i,j)=pH;
    end
  end
  fprintf(1,'  done.\n');
  figure(1)
  clf
  %V=10;
  %V=[100,150,200:100:800]; [c,h]=contour(DIC_axis,Alk_axis,CO2g_contour',V,'r');
  V=[8,11,15,20,30]; [c,h]=contour(DIC_axis,Alk_axis,H2CO3_contour',V,'r');
  clabel(c,h,'fontsize',14)
  set(gca,'xtick',min(DIC_axis):25:max(DIC_axis));
  set(gca,'ytick',min(Alk_axis):25:max(Alk_axis));
  hold on
  V=[7.8,8,8.2,8.4];
  [c,h]=contour(DIC_axis,Alk_axis,pH_contour',V,'--b');
  clabel(c,h,'fontsize',14)
  h1=xlabel('DIC');
  h2=ylabel('Alk');
  h3=title('pH (b) & CO2 (r)');
  set([gca,h1,h2,h3],'fontsize',14);
  axis square
  grid
  
  set(gcf, 'PaperUnits', 'inches'); set(gcf, 'PaperSize', [6 6]);
  set(gcf, 'PaperPosition', [0 0 6 6]); % [left, bottom, width, height];
  saveas(gcf,sprintf('Output/CO2g-as-func-of-DIC-and-Alk.pdf'));

  disp('pausing...')
  pause
  
 


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function alpha_w=alpha_wthr(t,p,pCO2,restart)
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%% Berner's (GEOCARB II paper, 1994) weathering feedback

if p.do_interactive_alpha_wthr==1
  %% Berner's nondimensional function definition:
  RCO2=pCO2/280;
  fB=(RCO2)^(0.09*p.Gamma_Berner)*(1+0.038*p.Gamma_Berner*log(RCO2))^(0.65);
  %% I am using it to calculate alpha_wthr:
  alpha_w=p.alpha_wthr0*fB;
  if alpha_w>1; alpha_w=1; elseif alpha_w<0; alpha_w=0; end
else
  %% use value from first time step:  (do this only when starting
  %% from steady state)
  alpha_w=restart.alpha_wthr;
end


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function write_restart(t,x,p,restart)
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
save(sprintf('Restart/restart-out-%2.2d.mat',p.scenario),'t','x','restart');


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function [t,x,restart]=read_restart(p)
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
  filename=sprintf('Restart/restart-in-%2.2d.mat',p.scenario);
  if exist(filename,'file')==0;
    disp(' ');
    fprintf(1,'\n *** restart file does not exist: %s \n\n',filename);
    x=NaN; t=NaN;
    restart.alpha_wthr=NaN; 
    restart.F_b_inorg=NaN; 
  else
    load(filename);
  end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%  
function p=pressure_effect_on_K(t,p)
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%% effect of pressure on dissociation and solubility product constants.
%% From Zeebe's csys3

%---------------------- Pressure effect on K's (Millero, 95) ----------%
TC=temperature(t,p); % Celsius!
T=TC+273.15;
P=p.K_sp_pressure;

RGAS = 8.314510;        % J mol-1 deg-1 (perfect Gas)  
R = 83.131;             % mol bar deg-1 
                        % conversion cm3 -> m3          *1.e-6
                        %            bar -> Pa = N m-2  *1.e+5
                        %                => *1.e-1 or *1/10

% index: K1 1, K2 2, Kb 3, Kw 4, Ks 5, Kf 6, Kspc 7, Kspa 8,
%        K1P 9, K2P 10, K3P 11

%----- note: there is an error in Table 9 of Millero, 1995.
%----- The coefficients -b0 and b1
%----- have to be multiplied by 1.e-3!

%----- there are some more errors! 
%----- the signs (+,-) of coefficients in Millero 95 do not
%----- agree with Millero 79


a0 = -[25.5   15.82  29.48  25.60  18.03    9.78  48.76   46. ...
	14.51 23.12 26.57];
a1 =  [0.1271 -0.0219 0.1622 0.2324 0.0466 -0.0090 0.5304  0.5304 ...
	0.1211 0.1758 0.2020];
a2 =  [0.0     0.0    2.608 -3.6246 0.316  -0.942  0.0     0.0 ...
	-0.321 -2.647 -3.042]*1.e-3;
b0 = -[3.08   -1.13   2.84   5.13   4.53    3.91  11.76   11.76 ...
	2.67 5.15 4.08]*1.e-3;
b1 =  [0.0877 -0.1475 0.0    0.0794 0.09    0.054  0.3692  0.3692 ...
	0.0427 0.09 0.0714]*1.e-3;
b2 =  [0.0     0.0    0.0    0.0    0.0     0.0    0.0     0.0 ...
	0.0 0.0 0.0];

for ipc=1:length(a0);
  deltav(ipc)  =  a0(ipc) + a1(ipc).*TC + a2(ipc).*TC.*TC;
  deltak(ipc)  = (b0(ipc) + b1(ipc).*TC + b2(ipc).*TC.*TC);  
  p.lnkpok0(ipc) = -(deltav(ipc)./(R.*T)).*P + (0.5*deltak(ipc)./(R.*T)).*P.*P;
end;

% K1 = K1*exp(p.lnkpok0(1));
% K2 = K2*exp(p.lnkpok0(2));
% Kb = Kb*exp(p.lnkpok0(3));
% Kw = Kw*exp(p.lnkpok0(4));
% Ks = Ks*exp(p.lnkpok0(5));
% Kf = Kf*exp(p.lnkpok0(6));
% Kspc = Kspc*exp(p.lnkpok0(7));
% Kspa = Kspa*exp(p.lnkpok0(8));
% K1P = K1P*exp(p.lnkpok0(9));
% K2P = K2P*exp(p.lnkpok0(10));
% K3P = K3P*exp(p.lnkpok0(11));


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function examine_oxygen_consequences(p)
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%% Calculate expected oxygen sources, sinks and accumulation

%% read data file, which contains both the o. (output) and
%% p. (parameters) structures:
load(sprintf('Output/history_files/out_scenario%2.2d.mat',p.scenario));

%% set up some arrays and variables:
Myr=1.e6;
b=1; e=o.nt;
time=o.t(b:e)/Myr;

%% First, calculate net organic carbon production anomaly rate in
%% Peta kg/year:
prod=o.F_prod(b:e);
remin=o.F_remin(b:e);
net_org_Carbon_prod_anomaly=(prod(b:e)-prod(1))-(remin(b:e)-remin(1));

%% plot carbon fluxes:
Myr=1.e6;
figure(5)
clf
subplot(3,1,1);
hl1=plot(time,prod,'r');
hold on
hl2=plot(time,remin,'g');
hl3=plot(time,net_org_Carbon_prod_anomaly,'b');
set_axis_limits(gca,time ...
                    ,[prod,remin,net_org_Carbon_prod_anomaly]);
set(gca,'xticklabel','');
h2=ylabel('10^{15}kg C/yr');
h3=title('(a) Carbon fluxes');
h4=legend('F'' prod O2','F'' remin O2','net','Location','West');
set([gca,h2,h3],'fontsize',12);
set([h4],'fontsize',9);
set([hl1,hl2,hl3],'linewidth',1.5)


%% plot O2 sources and sinks:
subplot(3,1,2);
% moles O2=weight C (gr)*(1 mole O2/mole C)/(12 gr C/mole C):
flux_O2_carbon=(net_org_Carbon_prod_anomaly*1.e18)*(1/12); 
%% next two fluxes need to be converted from MICRO moles per liter:
flux_O2_Fe2p_mixing=-o.F_Fe2p_mixing_ell(b:e)*1e-6*p.ocean_mass_kg*(2/8);
flux_O2_H2S_mixing=-o.F_H2S_mixing_ell(b:e)*1.e-6*p.ocean_mass_kg*2;
hl1=plot(time,flux_O2_carbon,'r');
hold on;
hl2=plot(time,flux_O2_Fe2p_mixing,'g');
hl3=plot(time,flux_O2_H2S_mixing,'b');
set_axis_limits(gca,time ...
                    ,[flux_O2_carbon,flux_O2_Fe2p_mixing,flux_O2_H2S_mixing]);
set(gca,'xticklabel','');
h2=ylabel('mole O_2/yr');
h3=title('(b) Oxygen sources and sinks');
h4=legend('C source','Fe sink','H_2S sink','Location','northWest');
set([gca,h2,h3],'fontsize',12);
set([h4],'fontsize',9);
set([hl1,hl2,hl3],'linewidth',1.5)


%% plot O2 accumulated and total net source:
subplot(3,1,3);
% fluxes in moles O2/yr
total_flux_O2=1e-12*(flux_O2_carbon+flux_O2_Fe2p_mixing+flux_O2_H2S_mixing); 
flux_O2_now=1e-12*(net_org_Carbon_prod_anomaly*0+15e12);
accumulated_O2=net_org_Carbon_prod_anomaly*0;
for t=2:o.nt
  accumulated_O2(t)=accumulated_O2(t-1)+total_flux_O2(t-1)*(time(t)-time(t-1))*Myr;
end
% translate accumulated O2 from moles to atmospehre (kg/cm^2) (where today's
% O2=0.21 atm, using (O2=0.032 kg/mole):
area1=4*pi*(6300e5)^2;
%accumulated_O2=accumulated_O2*0.032/area1;
[ax,hl1,hl2]=plotyy(time,accumulated_O2 ...
                    ,time,total_flux_O2);
% remove double tick marks on right y-axis: (reviewer 1 didn't like them...)
set(gca,'box','off'); %remove the box
set(ax(2),'XAxisLocation','top','linewidth',1,'XTickLabel',[]) %cover the top box manually
set(ax(2),'YTick',[]) %remove the ticks
% end of removing double tick marks
hl3=line(time,flux_O2_now,'color',[0 0.5 0],'linestyle','--','parent',ax(2));
set_axis_limits(ax(1),time,[accumulated_O2]);
set_axis_limits(ax(2),time,[total_flux_O2,flux_O2_now]);
h1=xlabel(ax(1),'time (Myr)');
set(ax(2),'xticklabel','');
xlabel(ax(2),' ');
h2a=ylabel(ax(1),'O_2 (moles)');
h2b=ylabel(ax(2),'10^{12}mole O_2/yr');
h3=title('(c) Oxygen total flux & accumulation');
h4=legend('accumulated O_2','net O_2 source','weathering flux','Location','West');
set([gca,h1,h2a,h2b,h3],'fontsize',12);
set([h4],'fontsize',9);
set([hl1,hl2,hl3],'linewidth',1.5)


%% save figure as pdf:
set(gcf, 'PaperUnits', 'inches'); set(gcf, 'PaperSize', [5 6]);
set(gcf, 'PaperPosition', [0 0 5 6]); % [left, bottom, width, height];
saveas(gcf,sprintf('Output/Figures_pdf/oxygen-consequences-scenario%2.2d.pdf',p.scenario));
if p.do_save_figures_as_Fig_files
  saveas(gcf,sprintf('Output/Figures_Fig_files/oxygen-consequences-scenario%2.2d.fig',p.scenario));
end


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function p=set_parameters(scenario)
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
  
  t=0;
  p.scenario=scenario;
  fprintf(1,'running scenario %2.2d\n',p.scenario);

  %% read scenario parameters:
  addpath Input;  % Could also put this line in my ~/matlab/startup.m
  eval(sprintf('eukaryotes_scenario%2.2d',p.scenario));

  %% add to p the effects of pressure on solubility and
  %% dissociation constants:
  p=pressure_effect_on_K(t,p);

  %% print some parameters to file to be used in a summary table in
  %% the paper:
  fid=fopen(sprintf('Output/Tables/table-%2.2d.tex',p.scenario),'w');
  fprintf(fid,'    Parameter & $U$   & $\\alpha$ & $\\beta$ & $\\gamma$ & $\\eta$ \\\\ \n');
  fprintf(fid,'    run %d     & %5.3g & %5.3g    & %5.3g   & %5.3g    & %5.3g \\\\ \n' ...
          ,p.scenario,p.Delta_ep,p.alpha_scenario ...
          ,p.beta_scenario,p.gamma_scenario,p.eta_scenario);
  fclose(fid);

  